
#include <ttcl/hj/dendrogram.hxx>
#include <ttcl/hj/ag/agglomerative_distance_clusterer.hxx>
#include <ttcl/hj/ag/single_link.hxx>
#include <ttcl/hj/ag/complete_link.hxx>
#include <ttcl/hj/ag/upgma.hxx>

#include <vector>
#include <iterator>
#include <map>

#include <cctype>


using namespace std;
using namespace ttcl;

/// --------------------------------------------------
/// Class example stores a set of pairs (atrib,value) 
/// plus an identifier for the example, and the 
/// assigned cluster

class example : public std::map<std::string,double> {
public:
  /// example identifier
  string id; 
  /// cluster the example belongs to
  int cluster;

  // constructor
  example(): id(""),cluster(-1) {};
  example(string s): id(s),cluster(-1) {};
};

/// --------------------------------------------------
/// distance measure between two examples

struct mydistance {

  typedef double result_type;

  double operator()(const example& a, const example& b) const {
    // compare features of example a and example b 
    // and compute distance

    // d = cosine(a,b)
    double nab=0, na=0, nb=0;

    for (map<string,double>::const_iterator fa=a.begin(); fa!=a.end(); fa++) {
      map<string,double>::const_iterator fb = b.find(fa->first);

      if (fb!=b.end()) {
	// for features in both vectors, add (a-b)^2
	nab += (fa->second - fb->second)*(fa->second - fb->second);
      }
      else {
	// for features only in a, add (a-0)^2
	na += fa->second*fa->second;
      }
    }

    // for features only in b, add (b-0)^2
    for (map<string,double>::const_iterator fb=b.begin(); fb!=b.end(); fb++) {
      if (a.find(fb->first)==a.end()) {
	nb += fb->second*fb->second;
      }
    }

    double d = na+nb+nab;
    return (sqrt(d));
  }

};

/// Load data colection and clusterize it.

int main(int argc, char* argv[]) {

  // read number of expected clusters
  int nclus=atoi(argv[1]);

  // load example colection
  vector<example> col;

  string line;
  while (getline(cin,line)) {
    istringstream sin; sin.str(line);
    
    // right class
    string clase;
    sin>>clase;

    // read pairs feature-value for current example
    double nrm=0;
    example ex(clase);
    string feat;
    double val;
    while ( sin>>feat>>val ) {
      ex.insert(make_pair(feat,val));
      nrm += val*val; // add v^2 to the norm
    }

    //compute vector norm and normalize vector components
    nrm=sqrt(nrm);
    for (example::iterator f=ex.begin(); f!=ex.end(); f++) 
      f->second=f->second/nrm;

    // add example to vector
    col.push_back(ex);
  }

  // create a clusterer. You can try using different linking strategies
  typedef ttcl::hj::ag::agglomerative_distance_clusterer<std::vector<example>,
                                                         mydistance, 
                                                         //ttcl::hj::ag::upgma  
                                                         //ttcl::hj::ag::single_link
                                                         ttcl::hj::ag::complete_link> clusterer_t;
  clusterer_t cl;

  // perform clustering
  clusterer_t::dendrogram_type d = cl.hier_cluster(col);

  // cut dendogram in desired number of clusters    
  cl::hard_clustering<> clus=d.cut(nclus);

  // output resulting cluster assignations (right class - assigned class)
  for (size_t i=0; i<col.size(); i++) {
    cout << col[i].id <<" "<< clus[i] <<endl;
  }

}
