#ifndef TTCL_ASSERT_HXX
#define TTCL_ASSERT_HXX

// TTCL: The Template Clustering Library

/** @file
    Assertions
    @author Edgar Gonzalez i Pellicer
*/

// Conditional includes
#ifdef TTCL_ASSERT
#include <ttcl/ut/printf_buffers.hxx>
#ifdef TTCL_ASSERT_WARN
#include <ttcl/lg/logger.hxx>
#else
#include <ttcl/exception.hxx>
#endif
#endif

/// Failed
#ifdef TTCL_ASSERT
#ifdef TTCL_ASSERT_WARN
#define ttcl_failed(...)			\
  ttcl_logger().warn(__VA_ARGS__);
#else
#define ttcl_failed(...)			\
  ttcl_fire(__VA_ARGS__);
#endif
#endif

/// General assertion
#ifdef TTCL_ASSERT
#define ttcl_assert(assertion)				\
  if (not (assertion))					\
    ttcl_failed("Assertion '%s' failed", #assertion);
#else
#define ttcl_assert(assertion)
#endif

/// Equal assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_eq(expression, value)				\
  if ((expression) != (value))						\
    ttcl_failed("Assertion '%s == %s' failed (%s = %s)",		\
		#expression, #value, #expression, ttcl_str(expression));
#else
#define ttcl_assert_eq(expression, value)
#endif

/// Not equal assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_ne(expression, value)				\
  if ((expression) == (value))						\
    ttcl_failed("Assertion '%s != %s' failed",				\
		#expression, #value);
#else
#define ttcl_assert_ne(expression, value)
#endif

/// Less than assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_lt(expression, value)				\
  if ((expression) >= (value))						\
    ttcl_failed("Assertion '%s < %s' failed (%s = %s)",			\
		#expression, #value, #expression, ttcl_str(expression));
#else
#define ttcl_assert_lt(expression, value)
#endif

/// Less than or equal assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_le(expression, value)				\
  if ((expression) > (value))						\
    ttcl_failed("Assertion '%s <= %s' failed (%s = %s)",		\
		#expression, #value, #expression, ttcl_str(expression));
#else
#define ttcl_assert_le(expression, value)
#endif

/// Greater than assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_gt(expression, value)				\
  if ((expression) <= (value))						\
    ttcl_failed("Assertion '%s > %s' failed (%s = %s)",			\
		#expression, #value, #expression, ttcl_str(expression));
#else
#define ttcl_assert_gt(expression, value)
#endif

/// Greater than or equal assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_ge(expression, value)				\
  if ((expression) < (value))						\
    ttcl_failed("Assertion '%s >= %s' failed (%s = %s)",		\
		#expression, #value, #expression, ttcl_str(expression));
#else
#define ttcl_assert_ge(expression, value)
#endif

/// Range assertion
#ifdef TTCL_ASSERT
#define ttcl_assert_range(lower, expression, upper)			\
  if ((expression) < (lower) or (expression) >= (upper))		\
    ttcl_failed("Assertion '%s <= %s < %s' failed (%s = %s)",		\
		#lower, #expression, #upper,				\
		#expression, ttcl_str(expression));
#else
#define ttcl_assert_range(lower, expression, upper)
#endif

#endif
