#ifndef TTCL_CL_HARD_DATA_CLUSTERER_HXX
#define TTCL_CL_HARD_DATA_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Hard data clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Hard data clusterer
    /** @param Data The data type
     */
    template <typename Data,
	      typename Choices = tu::default_choices>
    class hard_data_clusterer {
    public:
      /// Data type
      typedef Data data_type;

      /// Hard clustering container
      typedef ttcl_tu_get_choice(Choices, hard_clustering_container)
	hard_clustering_container_type;

      /// Hard clustering type
      typedef hard_clustering<hard_clustering_container_type>
        hard_clustering_type;

      /// Destructor
      virtual
      ~hard_data_clusterer() {
      }

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const data_type& _data,
		      const uint _n_clusters) const = 0;

      /// Perform the clustering
      /** Functional convenience version
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      hard_clustering_type
      hard_cluster(const data_type& _data,
		   const uint _n_clusters) const {
	// Build and return
	hard_clustering_type ret(_data.size(), _n_clusters);
	hard_cluster_on(ret, _data, _n_clusters);
	return ret;
      }
    };
  }
}

#endif
