#ifndef TTCL_CL_HARD_DATA_MODEL_HXX
#define TTCL_CL_HARD_DATA_MODEL_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Hard data model
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/tu/choices.hxx>


/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Hard data model
    /** @param Data The data type
     */
    template <typename Data,
	      typename Choices = tu::default_choices>
    class hard_data_model {
    public:
      /// Data type
      typedef Data data_type;

      /// Hard clustering container
      typedef ttcl_tu_get_choice(Choices, hard_clustering_container)
	hard_clustering_container_type;

      /// Hard clustering type
      typedef hard_clustering<hard_clustering_container_type>
        hard_clustering_type;

    protected:
      /// Number of clusters
      uint clusters_;

    public:
      /// Constructor
      hard_data_model(const uint _clusters = 0) :
	clusters_(_clusters) {
      }

      /// Destructor
      virtual
      ~hard_data_model() {
      }

      /// @name Setters/Getters
      /// @{

      /// Number of clusters
      uint clusters() const {
	return clusters_;
      }

      /// Set the number of clusters
      void set_clusters(const uint _clusters) {
	clusters_ = _clusters;
	resized();
      }

      /// Number of free parameters
      virtual uint
      free_parameters() const = 0;

      /// @}

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const data_type& _data) const = 0;

      /// Perform the clustering
      /** Functional convenience version
	  @param _data Data
      */
      hard_clustering_type
      hard_cluster(const data_type& _data) const {
	// Build and return
	hard_clustering_type ret(_data.size(), clusters_);
	hard_cluster_on(ret, _data);
	return ret;
      }

    protected:
      /// Resized
      /** Empty in this base class
       */
      virtual void resized() {
      }
    };
  }
}

#endif
