#ifndef TTCL_CL_HARD_DATA_MODEL_CLUSTERER_HXX
#define TTCL_CL_HARD_DATA_MODEL_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Hard model data clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/cl/hard_data_clusterer.hxx>
#include <ttcl/cl/hard_data_model.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Hard data model clusterer
    /** @param Data The data type
     */
    template <typename Data,
	      typename Choices = tu::default_choices>
    class hard_data_model_clusterer :
      public hard_data_clusterer<Data, Choices> {
    public:
      /// Base type
      typedef hard_data_clusterer<Data, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Hard clustering type
      typedef typename base_type::hard_clustering_type hard_clustering_type;

      /// Model type
      typedef typename hard_data_model<data_type, Choices> model_type;

      /// Create a model
      virtual model_type*
      create_model(const data_type& _data,
		   const uint _n_clusters) const = 0;

      /// Model
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual model_type*
      hard_model_on(hard_clustering_type& _hard_clustering,
		    const data_type& _data,
		    const uint _n_clusters) const = 0;

      /// Model
      /** @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual model_type *
      model(const data_type& _data,
	    const uint _n_clusters) const {
	// Cluster and return
	hard_clustering_type hard_clustering(_data.size(), _n_clusters);
	return hard_model_on(hard_clustering, _data, _n_clusters);
      }

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const data_type& _data,
		      const uint _n_clusters) const {
	// Cluster using a model, and ignore it
	model_type* ret = hard_model_on(_hard_clustering, _data, _n_clusters);
	delete ret;
      }
    };
  }
}

#endif
