#ifndef TTCL_CL_HARD_DISTANCE_MATRIX_CLUSTERER_HXX
#define TTCL_CL_HARD_DISTANCE_MATRIX_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Hard distance clusterer adapter
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/hard_distance_clusterer.hxx>
#include <ttcl/co/distance_matrix.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Hard distance clusterer adapter
    template <typename Data,
	      typename Distance,
	      typename MatrixClusterer,
	      typename Choices = tu::default_choices>
    class hard_distance_matrix_clusterer :
      public hard_distance_clusterer<Data, Distance, Choices> {
    public:
      /// Base type
      typedef distance_clusterer<Data, Distance, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Clustering type
      typedef typename base_type::hard_clustering_type hard_clustering_type;

      /// Distance type
      typedef typename base_type::distance_type distance_type;

      /// Matrix clusterer type
      typedef MatrixClusterer matrix_clusterer_type;

      /// Distance matrix type
      typedef typename matrix_clusterer_type::matrix_type matrix_type;

    protected:
      /// Base matrix clusterer
      matrix_clusterer_type matrix_clusterer_;

    public:
      /// Constructor
      hard_distance_matrix_clusterer(const distance_type& _distance,
				     const matrix_clusterer_type& _matrix_clusterer) :
	base_type(_distance),
	matrix_clusterer_(_matrix_clusterer) {
      }

      /// Perform the clustering
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const data_type& _data,
		      const uint _n_clusters = 1) const {
	// Create a distance matrix
	co::distance_matrix<matrix_type> distances(_data, this->distance_);

	// Call
	matrix_clusterer_.hard_cluster_on(_hard_clustering,
					  distances, _n_clusters);
      }
    };
  }
}

#endif
