#ifndef TTCL_CL_HARD_MATRIX_CLUSTERER_HXX
#define TTCL_CL_HARD_MATRIX_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Hard matrix clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/co/array.hxx>
#include <ttcl/co/symmetric_matrix.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Hard matrix clusterer
    /** @param Value The matrix value type
     */
    template <typename Matrix = co::symmetric_matrix< co::array<double> >,
	      typename Choices = tu::default_choices>
    class hard_matrix_clusterer {
    public:
      /// Distance matrix type
      typedef Matrix matrix_type;

      /// Value type
      typedef typename matrix_type::value_type value_type;

      /// Hard clustering container
      typedef ttcl_tu_get_choice(Choices, hard_clustering_container)
	hard_clustering_container_type;

      /// Clustering type
      typedef hard_clustering<hard_clustering_container_type>
        hard_clustering_type;

      /// Destructor
      virtual
      ~hard_matrix_clusterer() {
      }

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _in_distances Element pair-wise distances
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const matrix_type& _in_distances,
		      const uint _n_clusters) const = 0;

      /// Perform the hard clustering
      /** Functional convenience version
	  @param _in_distances Element pair-wise distances
	  @param _n_clusters Number of clusters
      */
      hard_clustering_type
      hard_cluster(const matrix_type& _in_distances,
		   const uint _n_clusters) const {
	// Build and return
	hard_clustering_type ret(_in_distances.size(), _n_clusters);
	hard_cluster_on(ret, _in_distances, _n_clusters);
	return ret;
      }
    };
  }
}

#endif
