#ifndef _TTCL_CL_SOFT_DATA_CLUSTERER
#define _TTCL_CL_SOFT_DATA_CLUSTERER

// TTCL: The Template Clustering Library

/** @file
    Clustering - Soft data clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/cl/hard_data_clusterer.hxx>
#include <ttcl/cl/soft_clustering.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Soft data clusterer
    /** @param Data The data type
     */
    template <typename Data,
	      typename Choices = tu::default_choices>
    class soft_data_clusterer :
      public hard_data_clusterer<Data, Choices> {
    public:
      /// Base type
      typedef hard_data_clusterer<Data, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Hard clustering type
      typedef typename base_type::hard_clustering_type hard_clustering_type;

      /// Soft clustering container
      typedef ttcl_tu_get_choice(Choices, soft_clustering_container)
	soft_clustering_container_type;

      /// Soft clustering type
      typedef soft_clustering<soft_clustering_container_type>
        soft_clustering_type;

      /// Perform the soft clustering
      /** @param _soft_clustering Target soft clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      soft_cluster_on(soft_clustering_type& _soft_clustering,
		      const data_type& _data,
		      const uint _n_clusters) const = 0;

      /// Perform the soft clustering
      /** Convenience functional version
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      soft_clustering_type
      soft_cluster(const data_type& _data,
		   const uint _n_clusters) const {
	// Build and return
	soft_clustering_type ret(_data.size(), _n_clusters);
	soft_cluster_on(ret, _data, _n_clusters);
	return ret;
      }

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const data_type& _data,
		      const uint _n_clusters) const {
	// Assign, cluster, assign
	soft_clustering_type _soft_clustering = _hard_clustering;
	soft_cluster_on(_soft_clustering, _data, _n_clusters);
	_hard_clustering = _soft_clustering;
      }
    };
  }
}



#endif
