#ifndef TTCL_CL_SOFT_DISTANCE_MATRIX_CLUSTERER_HXX
#define TTCL_CL_SOFT_DISTANCE_MATRIX_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Soft distance clusterer adapter
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/soft_distance_clusterer.hxx>
#include <ttcl/co/distance_matrix.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Soft distance clusterer adapter
    template <typename Data,
	      typename Distance,
	      typename MatrixClusterer,
	      typename Choices = tu::default_choices>
    class soft_distance_matrix_clusterer :
      public soft_distance_clusterer<Data, Distance, Choices> {
    public:
      /// Base type
      typedef distance_clusterer<Data, Distance, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Clustering type
      typedef typename base_type::soft_clustering_type soft_clustering_type;

      /// Distance type
      typedef typename base_type::distance_type distance_type;

      /// Matrix clusterer type
      typedef MatrixClusterer matrix_clusterer_type;

      /// Distance matrix type
      typedef typename matrix_clusterer_type::matrix_type matrix_type;

    protected:
      /// Base matrix clusterer
      matrix_clusterer_type matrix_clusterer_;

    public:
      /// Constructor
      soft_distance_matrix_clusterer(const distance_type& _distance,
				     const matrix_clusterer_type& _matrix_clusterer) :
	base_type(_distance),
	matrix_clusterer_(_matrix_clusterer) {
      }

      /// Perform the clustering
      virtual void
      soft_cluster_on(soft_clustering_type& _soft_clustering,
		      const data_type& _data,
		      const uint _n_clusters = 1) const {
	// Create a distance matrix
	co::distance_matrix<matrix_type> distances(_data, this->distance_);

	// Call
	matrix_clusterer_.soft_cluster_on(_soft_clustering,
					  distances, _n_clusters);
      }
    };
  }
}

#endif
