#ifndef TTCL_CL_SOFT_MATRIX_CLUSTERER_HXX
#define TTCL_CL_SOFT_MATRIX_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Soft matrix clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_matrix_clusterer.hxx>
#include <ttcl/cl/soft_clustering.hxx>
#include <ttcl/co/array.hxx>
#include <ttcl/co/symmetric_matrix.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Soft matrix clusterer
    /** @param Value The matrix value type
     */
    template <typename Matrix  = co::symmetric_matrix< co::array<double> >,
	      typename Choices = tu::default_choices>
    class soft_matrix_clusterer :
      public hard_matrix_clusterer<Matrix, Choices> {
    public:
      /// Base type
      typedef hard_matrix_clusterer<Matrix, Choices> base_type;

      /// Value type
      typedef typename base_type::value_type value_type;

      /// Distance matrix type
      typedef typename base_type::matrix_type matrix_type;

      /// Hard clustering type
      typedef typename base_type::hard_clustering_type hard_clustering_type;

      /// Soft clustering container
      typedef ttcl_tu_get_choice(Choices, soft_clustering_container)
	soft_clustering_container_type;

      /// Soft clustering type
      typedef typename soft_clustering<soft_clustering_container_type>
        soft_clustering_type;

      /// Perform the soft clustering
      /** @param _soft_clustering Target soft clustering
	  @param _in_distances Element pair-wise distances
	  @param _n_clusters Number of clusters
      */
      virtual void
      soft_cluster_on(soft_clustering_type& _soft_clustering,
		      const matrix_type& _in_distances,
		      const uint _n_clusters) const = 0;

      /// Perform the soft clustering
      /** Functional convenience version
	  @param _in_distances Element pair-wise distances
	  @param _n_clusters Number of clusters
      */
      soft_clustering_type
      soft_cluster(const matrix_type& _in_distances,
		   const uint _n_clusters) const {
	// Build and return
	soft_clustering_type ret(_in_distances.size(), _n_clusters);
	soft_cluster_on(ret, _in_distances, _n_clusters);
	return ret;
      }

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _in_distances Element pair-wise distances
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const matrix_type& _in_distances,
		      const uint _n_clusters) const {
	// Assign
	_hard_clustering = soft_cluster(_in_distances, _n_clusters);
      }
    };
  }
}

#endif
