#ifndef TTCL_CL_WR_MATRIX_WRAP_HXX
#define TTCL_CL_WR_MATRIX_WRAP_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Matrix wrapper
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/exception.hxx>
#include <ttcl/wr/interface_wrap.hxx>
#include <ttcl/wr/typed_shared_referee.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Wrappers Namespace
    namespace wr {

      /// Matrix referee
      template <typename T>
      class _matrix_referee :
	public ttcl::wr::typed_shared_referee<T> {
      public:
	/// Constructor
	_matrix_referee(T* _ptr = 0, bool _owned = true) :
	  ttcl::wr::typed_shared_referee<T>(_ptr, _owned) {
	}

	/// Get
	T* get() const {
	  return this->ptr_;
	}
      };

      /// Matrix wrapper
      class matrix_wrap :
	public  ttcl::wr::interface_wrap<> {
      public:
	/// Base type
	typedef ttcl::wr::interface_wrap<> base_type;
	
	/// Empty Constructor
	matrix_wrap() :
	  base_type() {
	}

	/// Constructor
	template <typename T>
	matrix_wrap(T* _value, bool _owned = true) :
	  base_type(new _matrix_referee<T>(_value, _owned)) {
	}

	/// Constructor
	template <typename T>
	matrix_wrap(T& _value, bool _owned = false) :
	  base_type(new _matrix_referee<T>(&_value, _owned)) {
	}

	/// Get!
	template <typename T>
	T* get() const {
	  const _matrix_referee<T>* tptr = 
	    dynamic_cast<const _matrix_referee<T>*>(ptr_.ptr());
	  if (not tptr)
	    ttcl_fire("Bad cast");
	  return tptr->get();
	}
      };
    }
  }
}

#endif
