#ifndef TTCL_CL_WR_SOFT_DATA_CLUSTERER_WRAP_HXX
#define TTCL_CL_WR_SOFT_DATA_CLUSTERER_WRAP_HXX

// TTCL: The Template Clustering Library

/** @file
    Clustering - Soft data clusterer wrapper
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/soft_clustering.hxx>
#include <ttcl/cl/wr/data_wrap.hxx>
#include <ttcl/cl/wr/hard_data_clusterer_wrap.hxx>
#include <ttcl/tu/choices.hxx>
#include <ttcl/wr/shared_ptr.hxx>
#include <ttcl/wr/shared_referee.hxx>
#include <ttcl/wr/typed_shared_referee.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Clustering Namespace
  namespace cl {

    /// Wrappers Namespace
    namespace wr {

      /// Soft data clusterer interface
      template <typename Choices>
      class _soft_data_clusterer_interface :
	public _hard_data_clusterer_interface<Choices> {
      public:
	/// Soft clustering container
	typedef ttcl_tu_get_choice(Choices, soft_clustering_container)
	  soft_clustering_container_type;

	/// Soft clustering type
	typedef soft_clustering<soft_clustering_container_type>
	soft_clustering_type;
	
	/// Constructor
	_soft_data_clusterer_interface(bool _owned = true,
				       uint _references = 0) :
	  _hard_data_clusterer_interface<Choices>(_owned, _references) {
	}
	
	/// Perform the soft clustering
	virtual void
	soft_cluster_on(soft_clustering_type& _soft_clustering,
			const data_wrap& _data,
			const uint _n_clusters) const = 0;

	/// Perform the clustering
	virtual soft_clustering_type
	soft_cluster(const data_wrap& _data,
		     const uint _n_clusters) const = 0;
      };

      /// Soft data clusterer referee
      template <typename T,
		typename Interface>
      class _soft_data_clusterer_referee :
	public _hard_data_clusterer_referee<T, Interface> {
      private:
	// Data type
	typedef typename T::data_type data_type;
	
	/// Soft clustering type
	typedef typename Interface::soft_clustering_type soft_clustering_type;

      public:
	/// Constructor
	_soft_data_clusterer_referee(T* _ptr = 0, bool _owned = true) :
	  _hard_data_clusterer_referee<T, Interface>(_ptr, _owned) {
	}

	/// Perform the soft clustering
	virtual void
	soft_cluster_on(soft_clustering_type& _soft_clustering,
			const data_wrap& _data,
			const uint _n_clusters) const {
	  // Convert to data type
	  const data_type* data = _data.get<data_type>();
	    
	  // Call
	  this->ptr_->soft_cluster_on(_soft_clustering, *data, _n_clusters);
	}

	/// Perform the clustering
	virtual soft_clustering_type
	soft_cluster(const data_wrap& _data,
		     const uint _n_clusters) const {
	  // Convert to data type
	  const data_type* data = _data.get<data_type>();
	  
	  // Call
	  return this->ptr_->soft_cluster(*data, _n_clusters);
	}
      };

      /// Base soft data clusterer wrapper
      template <typename Interface>
      class _soft_data_clusterer_wrap :
	public _hard_data_clusterer_wrap<Interface> {
      public:
	/// Soft clustering type
	typedef typename Interface::soft_clustering_type soft_clustering_type;

      protected:
	/// Empty Constructor
	_soft_data_clusterer_wrap() :
	  _hard_data_clusterer_wrap<Interface>() {
	}

	/// Constructor
	_soft_data_clusterer_wrap(Interface* _ptr) :
	  _hard_data_clusterer_wrap<Interface>(_ptr) {
	}

      public:
	/// Perform the soft clustering
	void
	soft_cluster_on(soft_clustering_type& _soft_clustering,
			const data_wrap& _data,
			const uint _n_clusters) const {
	  // Call
	  this->ptr_->soft_cluster_on(_soft_clustering, _data, _n_clusters);
	}

	/// Perform the clustering
	soft_clustering_type
	soft_cluster(const data_wrap& _data,
		     const uint _n_clusters) const {
	  // Call
	  return this->ptr_->soft_cluster(_data, _n_clusters);
	}
      };

      /// Soft data clusterer wrapper
      template <typename Choices = tu::default_choices>
      class soft_data_clusterer_wrap :
	public _soft_data_clusterer_wrap
          <_soft_data_clusterer_interface<Choices> > {
      private:
	/// Interface
	typedef _soft_data_clusterer_interface<Choices> interface;

	/// Base type
	typedef _soft_data_clusterer_wrap<interface> base_type;

      public:
	/// Empty Constructor
	soft_data_clusterer_wrap() :
	  base_type() {
	}

	/// Constructor
	template <typename T>
	soft_data_clusterer_wrap(T* _value, bool _owned = true) :
	  base_type(new _soft_data_clusterer_referee<T, interface>(_value,
								   _owned)) {
	}

	/// Constructor
	template <typename T>
	soft_data_clusterer_wrap(T& _value, bool _owned = false) :
	  base_type(new _soft_data_clusterer_referee<T, interface>(&_value,
								   _owned)) {
	}
      };
    }
  }
};

#endif
