#ifndef TTCL_CO__BASE_TREE_HXX
#define TTCL_CO__BASE_TREE_HXX

#ifndef TTCL_CO_SIDED_TREE_HXX
#ifndef TTCL_CO_TREE_HXX
#error "Cannot include _base_tree.hxx directly"
#endif
#endif

// TTCL: The Template Clustering Library

/** @file
    Containers - Base Tree
    @author Edgar Gonzalez i Pellicer
*/

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Tree
    template <typename Child>
    class _base_tree {
    public:
      /// Value type
      typedef Value value_type;

    protected:
      /// Information
      value_type info_;

      /// Parent
      Child* parent_;

      /// Next sibling
      Child* next_sibling_;

    public:
      /// Constructor
      _base_tree(const value_type& _info) :
	info_(_info), parent_(0), first_child_(0) {
      }

      /// Destructor
      ~_base_tree() {
	// Delete next sibling
	delete next_sibling_;
      }

      /// Accessor (const version)
      const value_type& operator*() const {
	return info_;
      }

      /// Accessor (non-const version)
      value_type& operator*() {
	return info_;
      }

      /// Accessor (const version)
      const value_type* operator->() const {
	return &info_;
      }

      /// Accessor (non-const version)
      value_type* operator->() {
	return &info_;
      }

      /// Parent (const version)
      const Child* parent() const {
	return parent_;
      }

      /// Parent (non-const version)
      Child* parent() {
	return parent_;
      }

      /// Next sibling (const version)
      const Child* next_sibling() const {
	return next_sibling_;
      }

      /// Next sibling (non-const version)
      Child* next_sibling() {
	return next_sibling_;
      }
    };
  }
}

#endif
