#ifndef TTCL_CO_DEFAULT_PAIR_HXX
#define TTCL_CO_DEFAULT_PAIR_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Default Pair
    @author Edgar Gonzlez i Pellicer
*/

#include <utility>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Pair with default first element
    template <typename T1, typename T2, int value>
    class default_pair_first :
      public std::pair<T1, T2> {
    public:
      /// Base type
      typedef std::pair<T1, T2> base_type;
     
      /// First type
      typedef typename base_type::first_type first_type;
      
      /// Second type
      typedef typename base_type::second_type second_type;

      /// Default value
      static const first_type default_value = value;

    public:
      /// Constructor
      default_pair_first(const second_type& _second = second_type()) :
	base_type(value, _second) {
      }

      /// Full constructor
      default_pair_first(const first_type& _first,
			 const second_type& _second) :
	base_type(_first, _second) {
      }
    };


    /// Pair with default second element
    template <typename T1, typename T2, int value>
    class default_pair_second :
      public std::pair<T1, T2> {
    public:
      /// Base type
      typedef std::pair<T1, T2> base_type;
     
      /// First type
      typedef typename base_type::first_type first_type;
      
      /// Second type
      typedef typename base_type::second_type second_type;

      /// Default value
      static const second_type default_value = second_type(value);

    public:
      /// Constructor
      default_pair_second(const first_type& _first = first_type(),
			  const second_type& _second = second_type(value)) :
	base_type(_first, _second) {
      }
    };
  }
}

// Static vars
#ifdef TTCL_DEBUG
#define ttcl_co_default_pair_static					\
  template <typename T1, typename T2, int value>			\
  const typename std::pair<T1,T2>::first_type				\
  ttcl::co::default_pair_first<T1,T2,value>::default_value;		\
  template <typename T1, typename T2, int value>			\
  const typename std::pair<T1,T2>::second_type				\
  ttcl::co::default_pair_second<T1,T2,value>::default_value
#else
#define ttcl_co_default_pair_static
#endif

#endif
