#ifndef TTCL_CO_DISTANCE_MATRIX_HXX
#define TTCL_CO_DISTANCE_MATRIX_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Distance matrix mixin
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/co/symmetric_matrix.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// A distance matrix mixin
    template <typename Base = symmetric_matrix<> >
    class distance_matrix : public Base {
    public:
      /// Base type
      typedef Base base_type;
      
      /// Iterator
      typedef typename base_type::iterator iterator;
      
      /// Constructor
      /** @param _data Data
	  @param _distance Distance
      */
      template <typename Data, typename Distance>
      distance_matrix(const Data& _data,
		      const Distance& _distance) :
	base_type(_data.size()) {
	// Fill the matrix
	iterator it_m = this->begin();
	for (typename Data::const_iterator it_i = _data.begin();
	     it_i != _data.end(); ++it_i)
	  for (typename Data::const_iterator it_j = _data.begin();
	       it_j != it_i; ++it_j)
	    *it_m++ = _distance(*it_i, *it_j);
      }
    };
  }
}

#endif
