#ifndef TTCL_CO_ITERATOR_PAIR_HXX
#define TTCL_CO_ITERATOR_PAIR_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Iterator pair
    @author Edgar Gonzlez i Pellicer
*/

#include <utility>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Iterator pair
    template <typename Iterator>
    class iterator_pair :
      public std::pair<Iterator, Iterator> {
    public:
      /// Iterator type
      typedef Iterator iterator;

      /// Empty constructor
      iterator_pair() {
      }

      /// Constructor
      iterator_pair(const iterator& _begin, const iterator& _end) :
	std::pair<iterator, iterator>(_begin, _end) {
      }

      /// Constructor from a raw pair
      iterator_pair(const std::pair<iterator, iterator>& _other) :
	std::pair<iterator, iterator>(_other) {
      }

      /// Constructor from another type
      template <typename Iterator2>
      iterator_pair(const iterator_pair<Iterator2>& _other) :
	std::pair<iterator, iterator>(_other.first, _other.second) {
      }

      /// Empty?
      bool empty() const {
	return this->first == this->second;
      }

      /// Begin
      iterator begin() const {
	return this->first;
      }

      /// End
      iterator end() const {
	return this->second;
      }
    };
  }
}

#endif
