#ifndef TTCL_CO_MATRIX_ADAPTER_HXX
#define TTCL_CO_MATRIX_ADAPTER_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Matrix adapter
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/co/array.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Matrix adapter
    template <typename Base = array<double> >
    class matrix_adapter : public Base {
    public:
      /// Base type
      typedef Base base_type;

      /// Value type
      typedef typename base_type::value_type value_type;

      /// Pointer type
      typedef typename base_type::pointer pointer;

      /// Const pointer type
      typedef typename base_type::const_pointer const_pointer;

      /// Size type
      typedef typename base_type::size_type size_type;

    private:
      /// Rows
      size_type rows_;

      /// Cols
      size_type cols_;

    public:
      /// Empty constructor
      matrix_adapter() :
	base_type(), rows_(), cols_() {
      }

      /// Constructor
      /** @param _rows  Rows
	  @param _cols  Columns
	  @param _value Value
      */
      matrix_adapter(size_type _rows,
		     size_type _cols,
		     const value_type& _value = value_type()) :
	base_type(_rows * _cols, _value),
	rows_(_rows), cols_(_cols) {
      }

      /// Rows
      size_type rows() const {
	return rows_;
      }

      /// Columns
      size_type columns() const {
	return cols_;
      }

      /// Raw resize
      void resize(size_type _rows, size_type _cols,
		  value_type _value = value_type()) {
	base_type::resize(_rows * _cols, _value);
	rows_ = _rows;
	cols_ = _cols;
      }

      /// Access
      /** Non-const version
       */
      pointer operator[](int _row) {
	return &(base_type::operator[](_row * cols_));
      }

      /// Access 
      /** const version
       */
      const_pointer operator[](int _row) const {
	return &(base_type::operator[](_row * cols_));
      }
    };
  }
}

#endif
