#ifndef TTCL_CO_SIDED_TREE_HXX
#define TTCL_CO_SIDED_TREE_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Sided Tree
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/co/_base_tree.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Sided Tree
    template <typename Value>
    class sided_tree :
      public _base_tree<sided_tree> {
    protected:
      /// Base type
      typedef _base_tree<sided_tree> base_type;

    public:
      /// Value type
      typedef typename base_type::value_type value_type;

    protected:
      /// First left child
      sided_tree* first_left_child_;

      /// Last left child
      sided_tree* last_left_child_;

      /// First right child
      sided_tree* first_right_child_;

      /// Last right child
      sided_tree* last_right_child_;

    public:
      /// Constructor
      sided_tree(const value_type& _info) :
	base_type(_info), first_left_child_(0), last_left_child_(0),
	first_right_child_(0), last_right_child_(0) {
      }

      /// Destructor
      ~sided_tree() {
	// Delete children
	delete first_left_child_;
	delete first_right_child_;
      }

      /// Add a left child
      void add_left_child(sided_tree* _new) {
	if (!first_left_child_) {
	  first_left_child_ = last_left_child_ = _new;
	}
	else {
	  last_left_child_->next_sibling_ = _new;
	  last_left_child_                = _new;
	}
	_new->parent_ = this;
      }

      /// Add a right child
      void add_right_child(sided_tree* _new) {
	if (!first_right_child_) {
	  first_right_child_ = last_right_child_ = _new;
	}
	else {
	  last_right_child_->next_sibling_ = _new;
	  last_right_child_                = _new;
	}
	_new->parent_ = this;
      }

      /// First left child (const version)
      const sided_tree* first_left_child() const {
	return first_left_child_;
      }

      /// First left child (non-const version)
      sided_tree* first_left_child() {
	return first_left_child_;
      }

      /// First right child (const version)
      const sided_tree* first_right_child() const {
	return first_right_child_;
      }

      /// First right child (non-const version)
      sided_tree* first_right_child() {
	return first_right_child_;
      }
    };

    /// Tree Preorder
    template <typename Value, typename Functor>
    void preorder(const sided_tree<Value>* _tree, const Functor& _functor) {
      _functor(_tree);
      if (_tree) {
	_functor(_tree->first_left_child());
	_functor(_tree->first_right_child());
	_functor(_tree->next_sibling());
      }
    }

    /// Tree inorder
    template <typename Value, typename Functor>
    void inorder(const sided_tree<Value>* _tree, const Functor& _functor) {
      if (_tree) {
	_functor(_tree->first_left_child());
      }
      _functor(_tree);
      if (_tree) {
	_functor(_tree->first_right_child());
	_functor(_tree->next_sibling());
      }
    }

    /// Tree Postorder
    template <typename Value, typename Functor>
    void postorder(const sided_tree<Value>* _tree, const Functor& _functor) {
      if (_tree) {
	_functor(_tree->first_left_child());
	_functor(_tree->first_right_child());
      }
      _functor(_tree);
      if (_tree) {
	_functor(_tree->next_sibling());
      }
    }
  }
}

#endif
