#ifndef TTCL_CO_TRAITS_HXX
#define TTCL_CO_TRAITS_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Traits
    @author Edgar Gonzalez i Pellicer
*/

#include <cstddef>
#include <utility>

#include <boost/type_traits/integral_constant.hpp>
#include <boost/type_traits/is_same.hpp>
#include <boost/utility/enable_if.hpp>

#include <ttcl/co/fake_pair.hxx>
#include <ttcl/ut/traits.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Has value type
    ttcl_has_type(has_value_type, value_type);

    /// Is sparse item
    template <typename T>
    struct is_sparse_item :
      public boost::false_type {
    };

    /// Is sparse item
    /** Specialization for std::pair
     */
    template <typename T1, typename T2>
    struct is_sparse_item< std::pair<T1, T2> > :
      public boost::true_type {
    };

    /// Is sparse item
    /** Specialization for ttcl::co::fake_pair_second
     */
    template <typename T1, typename T2, int V2>
    struct is_sparse_item< ttcl::co::fake_pair_second<T1, T2, V2> > :
      public boost::true_type {
    };


    /// Is sparse sample
    template <typename T, typename Enable = void>
    struct is_sparse_sample :
      public boost::false_type {
    };

    /// Is sparse sample
    template <typename T>
    struct is_sparse_sample
      <T, typename boost::enable_if< has_value_type<T> >::type> :
      public is_sparse_item<typename T::value_type> {
    };


    /// Is sparse container
    template <typename T, typename Enable = void>
    struct is_sparse_container :
      public boost::false_type {
    };
    
    /// Is sparse container
    template <typename T>
    struct is_sparse_container
      <T, typename boost::enable_if< has_value_type<T> >::type> :
      public is_sparse_sample<typename T::value_type> {
    };


    /// Signed version of a type
    template <typename T> struct signed_type {
      typedef T type;
    };

    /// Signed version of a type
    /** Specialization for char */
    template <> struct signed_type<unsigned char> {
      typedef char type;
    };

    /// Signed version of a type
    /** Specialization for short */
    template <> struct signed_type<unsigned short> {
      typedef short type;
    };

    /// Signed version of a type
    /** Specialization for int */
    template <> struct signed_type<unsigned int> {
      typedef int type;
    };

    /// Signed version of a type
    /** Specialization for long */
    template <>
    struct signed_type<unsigned long> {
      typedef long type;
    };

    /// Signed version of a type
    /** Specialization for long long */
    template <>
    struct signed_type<unsigned long long> {
      typedef long long type;
    };
  }
}

#endif
