#ifndef TTCL_CO_TREE_HXX
#define TTCL_CO_TREE_HXX

// TTCL: The Template Clustering Library

/** @file
    Containers - Tree
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/co/_base_tree.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Container Namespace
  namespace co {

    /// Tree
    template <typename Value>
    class tree :
      public _base_tree<tree> {
    protected:
      /// Base type
      typedef _base_tree<tree> base_type;

    public:
      /// Value type
      typedef typename base_type::value_type value_type;

    protected:
      /// First child
      tree* first_child_;

      /// Last child
      tree* last_child_;

    public:
      /// Constructor
      tree(const value_type& _info) :
	base_type(_info), first_child_(0), last_child_(0) {
      }

      /// Destructor
      ~tree() {
	// Delete children
	delete first_child_;
      }

      /// Add a child
      void add_child(tree* _new) {
	if (!first_child_) {
	  first_child_ = last_child_ = _new;
	}
	else {
	  last_child_->next_sibling_ = _new;
	  last_child_                = _new;
	}
	_new->parent_ = this;
      }

      /// First child (const version)
      const tree* first_child() const {
	return first_child_;
      }

      /// First child (non-const version)
      tree* first_child() {
	return first_child_;
      }
    };

    /// Tree Preorder
    template <typename Value, typename Functor>
    void preorder(const tree<Value>* _tree, const Functor& _functor) {
      _functor(_tree);
      if (_tree) {
	_functor(_tree->first_child());
	_functor(_tree->next_sibling());
      }
    }
    
    /// Tree Postorder
    template <typename Value, typename Functor>
    void postorder(const tree<Value>* _tree, const Functor& _functor) {
      if (_tree)
	_functor(_tree->first_child());
      _functor(_tree);
      if (_tree)
	_functor(_tree->next_sibling());
    }
  }
}

#endif
