#ifndef TTCL_CR_CALINSKI_MEDOID_SCORE_DATA_HXX
#define TTCL_CR_CALINSKI_MEDOID_SCORE_DATA_HXX

// TTCL: The Template Clustering Library

/** @file
    Criteria - Calinski-Harabasz Score - Medoids Based (For Matrix)
    @author Edgar Gonzlez i Pellicer
*/

#include <ttcl/cr/calinski_medoid_score_matrix.hxx>
#include <ttcl/cr/distance_matrix_adapter_criterion.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Criteria Namespace
  namespace cr {

    /// Calinski-Harabasz Score - Medoids Based (For Data)
    template <typename Data,
	      typename Dist,
	      typename Choices = tu::default_choices>
    class calinski_medoid_score_data :
      public distance_matrix_adapter_criterion<
        Data, Dist,
        calinski_medoid_score_matrix<
	  ttcl_tu_get_choice_tem(Choices, cl::distance_matrix_template,
				 typename Dist::result_type), Choices>,
        Choices> {
    public:
      /// Base type
      typedef distance_matrix_adapter_criterion<
        Data, Dist,
        calinski_medoid_score_matrix<
	  ttcl_tu_get_choice_tem(Choices, cl::distance_matrix_template,
				 typename Dist::result_type), Choices>,
          Choices> base_type;

      /// Distance type
      typedef typename base_type::distance_type distance_type;

      /// Matrix criterion type
      typedef typename base_type::matrix_criterion_type matrix_criterion_type;

      /// Constructor
      calinski_medoid_score_data(const distance_type& _distance =
				   distance_type(),
				 const matrix_criterion_type&
				   _matrix_criterion =
 				   matrix_criterion_type()) :
	base_type(_distance, _matrix_criterion) {
      }
    };
  }
}

#endif
