#ifndef TTCL_CR_DATA_CRITERION_HXX
#define TTCL_CR_DATA_CRITERION_HXX

// TTCL: The Template Clustering Library

/** @file
    Criteria - Data criterion
    @author Edgar Gonzlez i Pellicer
*/

#include <vector>

#include <ttcl/types.hxx>
#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/cr/choices.hxx>
#include <ttcl/hj/choices.hxx>
#include <ttcl/hj/dendrogram.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Criteria Namespace
  namespace cr {

    /// Data Criterion
    template <typename Data,
	      typename Choices = tu::default_choices>
    class data_criterion {
    public:
      /// Data type
      typedef Data data_type;

      /// Value type
      typedef ttcl_tu_get_choice(Choices, criterion_value_type)
	value_type;

      /// Hard clustering container
      typedef ttcl_tu_get_choice(Choices, cl::hard_clustering_container)
	hard_clustering_container_type;

      /// Hard Clustering type
      typedef cl::hard_clustering<hard_clustering_container_type>
        hard_clustering_type;

      /// Dendrogram value type
      typedef ttcl_tu_get_choice(Choices, hj::dendrogram_value)
	dendrogram_value_type;

      /// Dendrogram type
      typedef hj::dendrogram<dendrogram_value_type> dendrogram_type;

    public:
      /// Destructor
      virtual ~data_criterion() {
      }

      /// Find the criterion value
      virtual value_type
      hard_apply(const hard_clustering_type& _hard_clustering,
		 const data_type& _data) = 0;

      /// Find the criterion value for a tree
      /** Default non particularly efficient version
       */
      virtual std::vector<value_type>
      hier_apply(const dendrogram_type& _dendrogram,
		 const data_type& _data) {
	// Return value
	std::vector<value_type> ret(_data.size());

	// For each size
	hard_clustering_type hcl(_data.size());
	for (uint i = 1; i <= _data.size(); ++i) {
	  // Cut and calinski
	  _dendrogram.cut_on(hcl, i);
	  ret[i - 1] = this->hard_apply(hcl, _data);
	}

	// Return
	return ret;
      }
    };
  }
}

#endif
