#ifndef TTCL_CR_DISTANCE_DATA_CRITERION_HXX
#define TTCL_CR_DISTANCE_DATA_CRITERION_HXX

// TTCL: The Template Clustering Library

/** @file
    Criteria - Distance-based data criterion
    @author Edgar Gonzlez i Pellicer
*/

#include <vector>

#include <ttcl/cr/data_criterion.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Criteria Namespace
  namespace cr {

    /// Distance-based data criterion
    template <typename Data,
	      typename Dist,
	      typename Choices = tu::default_choices>
    class distance_data_criterion :
      public data_criterion<Data, Choices> {
    public:
      /// Base type
      typedef data_criterion<Data, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Distance type
      typedef Dist distance_type;

      /// Value type
      typedef typename base_type::value_type value_type;

      /// Hard Clustering type
      typedef typename base_type::hard_clustering_type hard_clustering_type;

      /// Dendrogram type
      typedef typename base_type::dendrogram_type dendrogram_type;

    protected:
      /// Distance
      distance_type distance_;

    public:
      /// Constructor
      distance_data_criterion(const distance_type& _distance =
			        distance_type()) :
	base_type(), distance_(_distance) {
      }
    };
  }
}

#endif
