#ifndef TTCL_CR_DISTANCE_MATRIX_ADAPTER_CRITERION_HXX
#define TTCL_CR_DISTANCE_MATRIX_ADAPTER_CRITERION_HXX

// TTCL: The Template Clustering Library

/** @file
    Criteria - Distance-to-matrix adapter criterion
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/co/distance_matrix.hxx>
#include <ttcl/cr/distance_data_criterion.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Criteria Namespace
  namespace cr {

    /// Distance-to-matrix adapter Criterion
    template <typename Data,
	      typename Dist,
	      typename MatrixCriterion,
	      typename Choices = tu::default_choices>
    class distance_matrix_adapter_criterion :
      public distance_data_criterion<Data, Dist, Choices> {
    public:
      /// Base type
      typedef distance_data_criterion<Data, Dist, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Distance type
      typedef typename base_type::distance_type distance_type;

      /// Value type
      typedef typename base_type::value_type value_type;

      /// Matrix criterion type
      typedef MatrixCriterion matrix_criterion_type;

      /// Distance matrix type
      typedef typename matrix_criterion_type::matrix_type matrix_type;

      /// Hard clustering type
      typedef typename matrix_criterion_type::hard_clustering_type
        hard_clustering_type;

      /// Dendrogram type
      typedef typename matrix_criterion_type::dendrogram_type dendrogram_type;


    protected:
      /// Base matrix criterion
      matrix_criterion_type matrix_criterion_;

    public:
      /// Constructor
      distance_matrix_adapter_criterion(const distance_type& _distance,
					const matrix_criterion_type&
					  _matrix_criterion) :
	base_type(_distance),
	matrix_criterion_(_matrix_criterion) {
      }

      /// Find the criterion value
      virtual value_type
      hard_apply(const hard_clustering_type& _hard_clustering,
		 const data_type& _data) {
	// Create a distance matrix
	co::distance_matrix<matrix_type> distances(_data, this->distance_);

	// Call
	return matrix_criterion_.hard_apply(_hard_clustering, distances);
      }

      /// Find the criterion value for a tree
      virtual std::vector<value_type>
      hier_apply(const dendrogram_type& _dendrogram,
		 const data_type& _data) {
	// Create a distance matrix
	co::distance_matrix<matrix_type> distances(_data, this->distance_);

	// Call
	return matrix_criterion_.hier_apply(_dendrogram, distances);
      }
    };
  }
}

#endif
