#ifndef TTCL_CR_MATRIX_CRITERION_HXX
#define TTCL_CR_MATRIX_CRITERION_HXX

// TTCL: The Template Clustering Library

/** @file
    Criteria - Matrix criterion
    @author Edgar Gonzlez i Pellicer
*/

#include <vector>

#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/hj/choices.hxx>
#include <ttcl/hj/dendrogram.hxx>
#include <ttcl/tu/choices.hxx>
#include <ttcl/types.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Criteria Namespace
  namespace cr {

    /// Matrix Criterion
    template <typename Matrix = co::symmetric_matrix< co::array<double> >,
	      typename Choices = tu::default_choices>
    class matrix_criterion {
    public:
      /// Distance matrix type
      typedef Matrix matrix_type;

      /// Value type
      typedef typename matrix_type::value_type value_type;

      /// Hard clustering container
      typedef ttcl_tu_get_choice(Choices, cl::hard_clustering_container)
	hard_clustering_container_type;

      /// Hard Clustering type
      typedef cl::hard_clustering<hard_clustering_container_type>
        hard_clustering_type;

      /// Dendrogram value type
      typedef ttcl_tu_get_choice(Choices, hj::dendrogram_value)
	dendrogram_value_type;

      /// Dendrogram type
      typedef hj::dendrogram<dendrogram_value_type> dendrogram_type;

    public:
      /// Destructor
      virtual ~matrix_criterion() {
      }

      /// Find the criterion value
      virtual value_type
      hard_apply(const hard_clustering_type& _hard_clustering,
		 const matrix_type& _matrix) = 0;

      /// Find the criterion value for a tree
      /** Default non particularly efficient version
       */
      virtual std::vector<value_type>
      hier_apply(const dendrogram_type& _dendrogram,
		 const matrix_type& _matrix) {
	// Return value
	std::vector<value_type> ret(_matrix.size());

	// For each size
	hard_clustering_type hcl(_matrix.size());
	for (uint i = 1; i <= _matrix.size(); ++i) {
	  // Cut and calinski
	  _dendrogram.cut_on(hcl, i);
	  ret[i - 1] = this->hard_apply(hcl, _matrix);
	}

	// Return
	return ret;
      }
    };
  }
}

#endif
