#ifndef TTCL_HJ_AGGLOMERATIVE_DISTANCE_CLUSTERER_HXX
#define TTCL_HJ_AGGLOMERATIVE_DISTANCE_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Hierarchical agglomerative clusterer distance version
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/choices.hxx>
#include <ttcl/hj/hier_distance_matrix_clusterer.hxx>
#include <ttcl/hj/ag/agglomerative_clusterer.hxx>
#include <ttcl/hj/ag/single_link.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Hierarchical Clustering Namespace
  namespace hj {

    /// Agglomerative Clustering Namespace
    namespace ag {

      /// Hierarchical agglomerative clusterer
      /** Distance version
       */
      template <typename Data,
		typename Dist,
		typename Policy = single_link,
		typename Choices = tu::default_choices>
      class agglomerative_distance_clusterer :
	public hier_distance_matrix_clusterer<
	  Data, Dist,
	  agglomerative_clusterer<
	    ttcl_tu_get_choice_tem(Choices, cl::distance_matrix_template,
				   typename Dist::result_type),
	    Policy, Choices>,
	  Choices> {
      public:
	/// Base type
	typedef hier_distance_matrix_clusterer<
	  Data, Dist,
	  agglomerative_clusterer<
	    ttcl_tu_get_choice_tem(Choices, cl::distance_matrix_template,
				   typename Dist::result_type),
	    Policy, Choices>,
	  Choices> base_type;

	/// Distance type
	typedef typename base_type::distance_type distance_type;

	/// Matrix clusterer type
	typedef typename base_type::matrix_clusterer_type matrix_clusterer_type;

	/// Policy type
	typedef typename matrix_clusterer_type::policy_type policy_type;
      
	/// Constructor
	agglomerative_distance_clusterer(const distance_type& _distance =
					   distance_type(),
					 const policy_type& _policy =
					   policy_type()) :
	  base_type(_distance, matrix_clusterer_type(_policy)) {
	}
      };
    }
  }
}

#endif
