#ifndef TTCL_HJ_AG_CHOICES_HXX
#define TTCL_HJ_AG_CHOICES_HXX

// TTCL: The Template Clustering Library

/** @file
    Hierarchical Clustering - Choices
    @author Edgar Gonzalez i Pellicer
*/

#include <cmath>
#include <functional>
#include <queue>
#include <vector>

#include <ttcl/co/array.hxx>
#include <ttcl/co/limit_priority_queue.hxx>
#include <ttcl/co/symmetric_matrix.hxx>
#include <ttcl/tu/choices.hxx>
#include <ttcl/tu/tuning.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Hierarchical Clustering Namespace
  namespace hj {

    /// Agglomerative Clustering Namespace
    namespace ag {

      /// STL Queue
      template <typename Value>
      class stl_queue :
	public std::priority_queue<Value,
				   std::vector<Value>,
				   std::greater<Value> > {
      };

      /// TTCL Limit Queue
      template <typename Value>
      class ttcl_limit_queue :
	public co::limit_priority_queue<Value,
					std::vector<Value>,
					std::greater<Value> > {
	/// Base type
	typedef co::limit_priority_queue<Value,
	  std::vector<Value>,
	  std::greater<Value> > base_type;
      
      public:
	/// Constructor
	ttcl_limit_queue(typename base_type::size_type _max_size) :
	  base_type(_max_size) {
	}
      };

      /// Default merging threshold
      const double default_merging_threshold = INFINITY;
      
      /// Merging threshold
      /** Option
       */
      ttcl_tu_option(double, merging_threshold, default_merging_threshold);

      /// Limit Queue Size
      /** Option
       */
      ttcl_tu_option(co::_limit_priority_queue::size_type, limit_queue_size,
		     co::_limit_priority_queue::default_max_size);

      /// Limit Queue Template
      /** Choice
       */
      ttcl_tu_choice_tem(limit_queue_template, ttcl_limit_queue);

      /// Queue Template
      /** Choice
       */
      ttcl_tu_choice_tem(queue_template, stl_queue);

    }
  }
}

#endif
