#ifndef TTCL_HJ_AG_SINGLE_LINK_HXX
#define TTCL_HJ_AG_SINGLE_LINK_HXX

// TTCL: The Template Clustering Library

/** @file
    Hierarchical Clustering - Single link policy
    @author Edgar Gonzalez i Pellicer
*/

#include <algorithm>

#include <ttcl/co/followable_mf_set.hxx>

/// TTCL Namespace
namespace ttcl {
  
  /// Hierarchical Clustering Namespace
  namespace hj {

    /// Agglomerative Clustering Namespace
    namespace ag {

      /// Single link policy
      class single_link {
      public:
	/// Update
	template <typename T>
	T operator()(const T _distance1, const T /* _size1 */,
		     const T _distance2, const T /* _size2 */) const {
	  return std::min(_distance1, _distance2);
	}

	/// Update
	template <typename Dist, typename Data>
	typename Dist::result_type
	operator()(const Dist& _distance,
		   const Data& _data,
		   const co::followable_mf_set::const_follow_iterator& _begin1,
		   const co::followable_mf_set::const_follow_iterator& _end1,
		   const co::followable_mf_set::const_follow_iterator& _begin2,
		   const co::followable_mf_set::const_follow_iterator& _end2) const {
	  // Minimum
	  typename Dist::result_type ret =
	    _distance(_data[*_begin1], _data[*_begin2]);
	  for (co::followable_mf_set::const_follow_iterator it1 = _begin1;
	       it1 != _end1; ++it1) {
	    for (co::followable_mf_set::const_follow_iterator it2 = _begin2;
		 it2 != _end2; ++it2) {
	      typename Dist::result_type val =
		_distance(_data[*it1], _data[*it2]);
	      if (val < ret) ret = val;
	    }
	  }

	  // Return the minimum
	  return ret;
	}
      };
    }
  }
}

#endif
