#ifndef TTCL_HJ_AG_UPGMA_HXX
#define TTCL_HJ_AG_UPGMA_HXX

// TTCL: The Template Clustering Library

/** @file
    Hierarchical Clustering - UPGMA Policy
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/types.hxx>
#include <ttcl/co/followable_mf_set.hxx>

/// TTCL Namespace
namespace ttcl {
  
  /// Hierarchical Clustering Namespace
  namespace hj {

    /// Agglomerative Clustering Namespace
    namespace ag {

      /// UPGMA policy
      class upgma {
      public:
	/// Update
	template <typename T>
	T operator()(const T _distance1, const T _size1,
		     const T _distance2, const T _size2) const {
	  return
	    (_distance1 * _size1 + _distance2 * _size2) / (_size1 + _size2);
	}

	/// Update
	template <typename Dist, typename Data>
	typename Dist::result_type
	operator()(const Dist& _distance,
		   const Data& _data,
		   const co::followable_mf_set::const_follow_iterator& _begin1,
		   const co::followable_mf_set::const_follow_iterator& _end1,
		   const co::followable_mf_set::const_follow_iterator& _begin2,
		   const co::followable_mf_set::const_follow_iterator& _end2) const {
	  // Sum and count
	  typedef typename Dist::result_type value_type;
	  value_type sum = value_type();
	  uint size = 0;
	  for (co::followable_mf_set::const_follow_iterator it1 = _begin1;
	       it1 != _end1; ++it1) {
	    for (co::followable_mf_set::const_follow_iterator it2 = _begin2;
		 it2 != _end2; ++it2) {
	      sum += _distance(_data[*it1], _data[*it2]);
	      ++size;
	    }
	  }

	  // Return the average
	  return sum / value_type(size);
	}
      };
    }
  }
}

#endif
