#ifndef TTCL_HJ_HIER_DATA_CLUSTERER_HXX
#define TTCL_HJ_HIER_DATA_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Hierarchical Clustering - Data clusterer base class
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/cl/hard_data_clusterer.hxx>
#include <ttcl/hj/choices.hxx>
#include <ttcl/hj/dendrogram.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Hierarchical Clustering Namespace
  namespace hj {

    /// Hierarchical data clusterer base class
    template <typename Data,
	      typename Choices = ttcl::tu::default_choices>
    class hier_data_clusterer :
      public cl::hard_data_clusterer<Data, Choices> {
    public:
      /// Base type
      typedef cl::hard_data_clusterer<Data, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Clustering type
      typedef typename base_type::hard_clustering_type hard_clustering_type;

      /// Value type
      typedef ttcl_tu_get_choice(Choices, dendrogram_value) value_type;

      /// Dendrogram type
      typedef dendrogram<value_type> dendrogram_type;

      /// Destructor
      virtual
      ~hier_data_clusterer() {
      }

      /// Perform the clustering
      /** @param _dendrogram Target dendrogram
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hier_cluster_on(dendrogram_type& _dendrogram,
		      const data_type& _data,
		      const uint _n_clusters = 1) const = 0;

      /// Perform the clustering
      /** Functional convenience version
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      dendrogram_type
      hier_cluster(const data_type& _data,
		   const uint _n_clusters = 1) const {
	// Build and return
	dendrogram_type ret;
	hier_cluster_on(ret, _data, _n_clusters);
	return ret;
      }

      /// Perform the hard clustering
      /** @param _hard_clustering Target hard clustering
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hard_cluster_on(hard_clustering_type& _hard_clustering,
		      const data_type& _data,
		      const uint _n_clusters) const {
	// Build the dendrogram
	dendrogram_type dendro;
	hier_cluster_on(dendro, _data, _n_clusters);

	// Return it cut
	dendro.cut_on(_hard_clustering, _n_clusters);
      }
    };
  }
}

#endif
      
