#ifndef TTCL_HJ_DISTANCE_MATRIX_CLUSTERER_HXX
#define TTCL_HJ_DISTANCE_MATRIX_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Hierarchical Clustering - Distance clusterer adapter
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/co/distance_matrix.hxx>
#include <ttcl/hj/hier_distance_clusterer.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Hierarchical Clustering Namespace
  namespace hj {

    /// Hiearchical distance clusterer adapter
    /** Requirements:
	- dendrogram_type must be the same as
	  MatrixClusterer::dendrogram_type
     */
    template <typename Data,
	      typename Dist,
	      typename MatrixClusterer,
	      typename Choices = tu::default_choices>
    class hier_distance_matrix_clusterer :
      public hier_distance_clusterer<Data, Dist, Choices> {
    public:
      /// Base type
      typedef hier_distance_clusterer<Data, Dist, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Dendrogram type
      typedef typename base_type::dendrogram_type dendrogram_type;

      /// Distance type
      typedef typename base_type::distance_type distance_type;

      /// Matrix clusterer type
      typedef MatrixClusterer matrix_clusterer_type;

      /// Distance matrix type
      typedef typename matrix_clusterer_type::matrix_type matrix_type;

    protected:
      /// Base matrix clusterer
      matrix_clusterer_type matrix_clusterer_;

    public:
      /// Constructor
      hier_distance_matrix_clusterer(const distance_type& _distance,
				     const matrix_clusterer_type&
				       _matrix_clusterer) :
	base_type(_distance), matrix_clusterer_(_matrix_clusterer) {
      }

      /// Perform the clustering
      /** @param _dendrogram Target dendrogram
	  @param _data Data
	  @param _n_clusters Number of clusters
      */
      virtual void
      hier_cluster_on(dendrogram_type& _dendrogram,
		      const data_type& _data,
		      const uint _n_clusters = 1) const {
	// Create a distance matrix
	co::distance_matrix<matrix_type> distances(_data, this->distance_);

	// Call
	matrix_clusterer_.hier_cluster_inplace_on(_dendrogram,
						  distances, _n_clusters);
      }
    };
  }
}

#endif
