#ifndef TTCL_IT_BLOCK_DIAGONAL_CLUSTERER_HXX
#define TTCL_IT_BLOCK_DIAGONAL_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    Iterative Algorithms - Block Diagonal Clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/types.hxx>
#include <ttcl/it/block_diagonal_model.hxx>
#include <ttcl/it/hard_iterative_clusterer.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {
    
    /// Block Diagonal Clusterer
    template <typename Data,
	      typename Choices = tu::default_choices>
    class block_diagonal_clusterer :
      public hard_iterative_clusterer<Data, Choices> {
    public:
      /// Base type
      typedef hard_iterative_clusterer<Data, Choices> base_type;

      /// Data type
      typedef typename base_type::data_type data_type;

      /// Random generator type
      typedef typename base_type::random_generator_type random_generator_type;

      /// Model type
      typedef block_diagonal_model<Data, Choices> model_type;

      /// Base model type
      typedef typename base_type::model_type base_model_type;

      /// Constructor
      block_diagonal_clusterer(const random_generator_type&
			         _random_generator = 
			       random_generator_type()) :
	base_type(_random_generator) {
      }

      /// Create a model
      virtual model_type*
      create_model(const data_type& _data,
		   const uint _n_clusters) const {
	uint n_features = _data.features();
	return new model_type(_n_clusters, n_features);
      }
    };
  }
}

#endif
