#ifndef TTCL_IT_CHOICES_HXX
#define TTCL_IT_CHOICES_HXX

// TTCL: The Template Clustering Library

/** @file
    Iterative Algorithms - Choices
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/tu/choices.hxx>
#include <ttcl/tu/tuning.hxx>
#include <ttcl/types.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Default maximum iterations
    const uint default_max_iterations = 100;

    /// Default stop threshold
    const double default_stop_threshold = 1e-10;

    /// Start policies
    enum start_policy_enum { START_CENTROIDS, START_PEAKS, START_RANDOM,
			     DEFAULT_START = START_RANDOM };

    /// Default peak fraction
    const double default_peak_fraction = 0.001;
 
    /// Maximum iterations
    /** Option
     */
    ttcl_tu_option(uint, max_iterations, default_max_iterations);

    /// Stop threshold
    /** Option
     */
    ttcl_tu_option(double, stop_threshold, default_stop_threshold); 

    /// Start policy
    /** Option
     */
    ttcl_tu_option(start_policy_enum, start_policy, DEFAULT_START);

    /// Peak fraction
    /** Option
     */
    ttcl_tu_option(double, peak_fraction, default_peak_fraction);

    /// Model value type
    /** Choice
     */
    ttcl_tu_choice(model_value_type, double); 
  }
}

#endif
