#ifndef TTCL_IT_EM__CENTROIDS_MIXIN_HXX
#define TTCL_IT_EM__CENTROIDS_MIXIN_HXX

#ifndef TTCL_IT_EM_ALPHA_CENTROIDS_MODEL_HXX
#ifndef TTCL_IT_EM_CENTROIDS_MODEL_HXX
#error "Cannot include _centroids_mixin.hxx directly"
#endif
#endif

// TTCL: The Template Clustering Library

/** @file
    EM - Centroids Mixin
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/types.hxx>
#include <ttcl/co/centroid_set.hxx>
#include <ttcl/co/choices.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Expectation-Maximization Namespace
    namespace em {

      /// Centroids mixin
      template <typename Base, typename Choices = ttcl::tu::default_choices>
      class _centroids_mixin :
	public Base {
      public:
	/// Base type
	typedef Base base_type;

	/// Value type
	typedef typename base_type::value_type value_type;

	/// Parameter allocator type
	typedef ttcl_tu_get_choice_tem(Choices, co::parameter_allocator,
				       value_type) parameter_allocator_type;

	/// Centroids type
	typedef co::centroid_set<value_type, parameter_allocator_type>
	  centroids_type;

	/// Centroids type
	typedef typename centroids_type::centroid_type centroid_type;

      protected:
	/// Number of features
	uint features_;

	/// Centroids
	centroids_type centroids_;

      public:
	/// Empty Constructor
	_centroids_mixin() :
	  base_type(), features_(0), centroids_() {
	}

	/// Constructor
	_centroids_mixin(const uint _clusters, const uint _features) :
	  base_type(_clusters), features_(_features),
	  centroids_(_clusters, _features) {
	}

	/// Copy constructor
	_centroids_mixin(const _centroids_mixin& _other) :
	  base_type(_other), features_(_other.features_),
	  centroids_(_other.centroids_) {
	}

	/// @name Setters/Getters
	/// @{

	/// Number of features
	uint features() const {
	  return features_;
	}

	/// Set the number of features
	void set_features(uint _features) {
	  features_ = _features;
	  resized();
	}

	/// Centroid coordinate
	value_type centroid_coordinate(uint _cluster, uint _feature) const {
	  return centroids_[_cluster][_feature];
	}

	/// Set a centroid coordinate
	void set_centroid_coordinate(uint _cluster, uint _feature,
				     value_type _value) {
	  centroids_[_cluster][_feature] = _value;
	}

	/// @}

      protected:
	/// Resized
	virtual void resized() {
	  // Base
	  base_type::resized();

	  // Resize centroids
	  centroids_.resize(this->clusters_, features_);
	}
      };
    }
  }
}

#endif
