#ifndef TTCL_IT_EM_ALPHA_BREGMAN_CLUSTERER_HXX
#define TTCL_IT_EM_ALPHA_BREGMAN_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    EM - Bregman-Divergence Based Clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/it/em/choices.hxx>
#include <ttcl/it/em/em_clusterer.hxx>
#include <ttcl/it/em/alpha_bregman_model.hxx>
#include <ttcl/tu/choices.hxx>
#include <ttcl/tu/tuning.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Expectation-Maximization Namespace
    namespace em {

      /// Bregman-Divergence Based Clusterer
      template <typename Data,
		typename Divergence,
		typename Choices = tu::default_choices>
      class alpha_bregman_clusterer :
	public em_clusterer<Data, Choices> {
      public:
	/// Base type
	typedef em_clusterer<Data, Choices> base_type;

	/// Data type
	typedef typename base_type::data_type data_type;

	/// Value type
	typedef typename base_type::value_type value_type;

	/// Random generator type
	typedef typename base_type::random_generator_type random_generator_type;

	/// Divergence type
	typedef Divergence divergence_type;

	/// Model type
	typedef alpha_bregman_model<data_type, divergence_type, Choices>
	  model_type;

	/// Base model type
	typedef typename base_type::model_type base_model_type;

      protected:
	/// Alpha laplace term
	value_type alpha_laplace_term_;

	/// Divergence
	divergence_type divergence_;

      public:
	/// Constructor
	alpha_bregman_clusterer(const divergence_type& _divergence =
				  divergence_type(),
				const random_generator_type& _random_generator =
				  random_generator_type()) :
	  base_type(_random_generator),
	  alpha_laplace_term_(default_laplace_term),
	  divergence_(_divergence) {
	}

	/// Tune
	virtual void
	tune(const tu::tuning& _tuning) {
	  // Call parent version
	  base_type::tune(_tuning);

	  // Extract the options
	  alpha_laplace_term_ = _tuning.get<alpha_laplace_term>();
	}

	/// Create a model
	virtual model_type*
	create_model(const data_type& _data,
		     const uint _n_clusters) const {
	  uint n_features = _data.features();
	  return new model_type(_n_clusters, n_features, divergence_,
				alpha_laplace_term_);
	}
      };
    }
  }
}

#endif
