#ifndef TTCL_IT_EM_ALPHA_CENTROIDS_MODEL_HXX
#define TTCL_IT_EM_ALPHA_CENTROIDS_MODEL_HXX

// TTCL: The Template Clustering Library

/** @file
    EM - Alpha-Centroids Model
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/types.hxx>
#include <ttcl/it/em/alpha_model.hxx>
#include <ttcl/it/em/_centroids_mixin.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Expectation-Maximization Namespace
    namespace em {

      /// Alpha-Centroids Model
      template <typename Data,
		typename Choices = tu::default_choices>
      class alpha_centroids_model :
	public _centroids_mixin< alpha_model<Data, Choices>, Choices > {
      public:
	/// Base type
	typedef _centroids_mixin< alpha_model<Data, Choices>, Choices >
	  base_type;

	/// Value type
	typedef typename base_type::value_type value_type;

	/// Empty Constructor
	alpha_centroids_model() :
	  base_type() {
	}

	/// Constructor
	alpha_centroids_model(const uint _clusters, const uint _features,
			      const value_type _alpha_laplace_term =
			        default_laplace_term) :
	  base_type(_clusters, _features) {
	  // A quirk to be able to reuse _centroids_mixin
	  this->alpha_laplace_term_ = _alpha_laplace_term;
	}

	/// Copy constructor
	alpha_centroids_model(const alpha_centroids_model& _other) :
	  base_type(_other) {
	}

	/// Number of free parameters
	virtual uint
	free_parameters() const {
	  return this->clusters_ * (this->features_ + 1);
	}
      };
    }
  }
}

#endif
