#ifndef TTCL_IT_EM_CENTROIDS_MODEL_HXX
#define TTCL_IT_EM_CENTROIDS_MODEL_HXX

// TTCL: The Template Clustering Library

/** @file
    EM - Centroids Model
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/types.hxx>
#include <ttcl/co/centroid_set.hxx>
#include <ttcl/co/choices.hxx>
#include <ttcl/it/em/_centroids_mixin.hxx>
#include <ttcl/it/em/em_model.hxx>
#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Expectation-Maximization Namespace
    namespace em {

      /// Centroids Model
      template <typename Data,
		typename Choices = tu::default_choices>
      class centroids_model :
	public _centroids_mixin< em_model<Data, Choices>, Choices > {
      public:
	/// Base type
	typedef _centroids_mixin< em_model<Data, Choices>, Choices > base_type;

	/// Empty Constructor
	centroids_model() :
	  base_type() {
	}

	/// Constructor
	centroids_model(const uint _clusters, const uint _features) :
	  base_type(_clusters, _features) {
	}

	/// Copy constructor
	centroids_model(const centroids_model& _other) :
	  base_type(_other) {
	}

	/// Number of free parameters
	virtual uint
	free_parameters() const {
	  return this->clusters_ * this->features_;
	}
      };
    }
  }
}

#endif
