#ifndef TTCL_IT_EM_CHOICES_HXX
#define TTCL_IT_EM_CHOICES_HXX

// TTCL: The Template Clustering Library

/** @file
    EM - Choices
    @author Edgar Gonzalez i Pellicer
*/

#include <memory>

#include <ttcl/tu/tuning.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Expectation-Maximization Namespace
    namespace em {

      /// Default laplace term
      const double default_laplace_term = 1.0;

      /// Default noisy term
      const double default_noisy_term = 1.0;

      /// Default learning rate
      const double default_learning_rate = 0.2;

      /// Default starting prior
      const double default_starting_prior = 1000.0;

      /// Default prior scaling
      const double default_prior_scaling = 0.5;

      /// Default prior threshold
      const double default_prior_threshold = 1e-3;

      /// Softness
      enum softness_enum { SOFTNESS_HARD, SOFTNESS_SOFT,
			   DEFAULT_SOFTNESS = SOFTNESS_SOFT };

      /// Alpha laplace term
      /** Option
       */
      ttcl_tu_option(double, alpha_laplace_term, default_laplace_term);

      /// Theta laplace term
      /** Option
       */
      ttcl_tu_option(double, theta_laplace_term, default_laplace_term);

      /// Mean laplace term
      /** Option
       */
      ttcl_tu_option(double, mean_laplace_term, default_laplace_term);

      /// Variance laplace term
      /** Option
       */
      ttcl_tu_option(double, variance_laplace_term, default_laplace_term);

      /// Gaussian noisy term
      /** Option
       */
      ttcl_tu_option(double, gaussian_noisy_term, default_noisy_term);

      /// Learning rate
      /** Option
       */
      ttcl_tu_option(double, learning_rate, default_learning_rate);

      /// Starting prior
      /** Option
       */
      ttcl_tu_option(double, starting_prior, default_starting_prior);

      /// Prior scaling
      /** Option
       */
      ttcl_tu_option(double, prior_scaling, default_prior_scaling);

      /// Prior threshold
      /** Option
       */
      ttcl_tu_option(double, prior_threshold, default_prior_threshold);

      /// Inner softness
      /** Option
       */
      ttcl_tu_option(softness_enum, inner_softness, DEFAULT_SOFTNESS);
    }
  }
}

#endif
