#ifndef TTCL_IT_EM_NB_BERNOUILLI_NOISY_CLUSTERER_HXX
#define TTCL_IT_EM_NB_BERNOUILLI_NOISY_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    EM - Bernouilli Naive Bayes Noisy Clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <ttcl/it/em/nb_bernouilli_noisy_model.hxx>
#include <ttcl/it/em/nb_bernouilli_clusterer.hxx>
#include <ttcl/it/em/noisy_clusterer.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {

    /// Expectation-Maximization Namespace
    namespace em {

      /// Naive-Bayes Clusterer
      template <typename Data,
		typename Choices = tu::default_choices>
      class nb_bernouilli_noisy_clusterer :
	public noisy_clusterer< nb_bernouilli_clusterer<Data, Choices> > {
      public:
	/// Base type
	typedef noisy_clusterer< nb_bernouilli_clusterer<Data, Choices> >
	  base_type;

	/// Data type
	typedef typename base_type::data_type data_type;

	/// Random generator type
	typedef typename base_type::random_generator_type random_generator_type;

	/// Model type
	typedef nb_bernouilli_noisy_model<data_type, Choices> model_type;

	/// Base model type
	typedef typename base_type::base_model_type base_model_type;

	/// Constructor
	nb_bernouilli_noisy_clusterer(const random_generator_type&
				        _random_generator =
				        random_generator_type()) :
	  base_type(_random_generator) {
	}

	/// Create a model
	virtual model_type*
	create_model(const data_type& _data,
		     const uint _n_clusters) const {
	  uint n_features = _data.features();
	  return new model_type(_n_clusters, n_features,
				this->alpha_laplace_term_,
				this->theta_laplace_term_);
	}
      };
    }
  }
}

#endif
