#ifndef TTCL_IT_EM_NOISY_CLUSTERER_HXX
#define TTCL_IT_EM_NOISY_CLUSTERER_HXX

// TTCL: The Template Clustering Library

/** @file
    EM - Noisy Clusterer Mixin
    @author Edgar Gonzalez i Pellicer
*/

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {
    
    /// Expectation-Maximization Namespace
    namespace em {

      /// Noisy Clusterer Mixin
      template <typename Base>
      class noisy_clusterer :
	public Base {
      public:
	/// Base type
	typedef Base base_type;

	/// Value type
	typedef typename base_type::value_type value_type;

	/// Soft clustering type
	typedef typename base_type::soft_clustering_type soft_clustering_type;

	/// Propagated one-parameter constructor
	template <typename T1>
	noisy_clusterer(const T1& _v1) :
	  base_type(_v1) {
	}

	/// Propagated two-parameter constructor
	template <typename T1, typename T2>
	noisy_clusterer(const T1& _v1, const T2& _v2) :
	  base_type(_v1, _v2) {
	}

	/// Propagated three-parameter constructor
	template <typename T1, typename T2, typename T3>
	noisy_clusterer(const T1& _v1, const T2& _v2, const T3& _v3) :
	  base_type(_v1, _v2, _v3) {
	}

	/// Propagated four-parameter constructor
	template <typename T1, typename T2, typename T3, typename T4>
	noisy_clusterer(const T1& _v1, const T2& _v2, const T3& _v3,
			const T4& _v4) :
	  base_type(_v1, _v2, _v3, _v4) {
	}

	/// Propagated five-parameter constructor
	template <typename T1, typename T2, typename T3, typename T4,
		  typename T5>
	noisy_clusterer(const T1& _v1, const T2& _v2, const T3& _v3,
			const T4& _v4, const T5& _v5) :
	  base_type(_v1, _v2, _v3, _v4, _v5) {
	}

	/// Propagated six-parameter constructor
	template <typename T1, typename T2, typename T3, typename T4,
		  typename T5, typename T6>
	noisy_clusterer(const T1& _v1, const T2& _v2, const T3& _v3,
			const T4& _v4, const T5& _v5, const T6& _v6) :
	  base_type(_v1, _v2, _v3, _v4, _v5, _v6) {
	}

	/// Randomize the clustering
	/** Include a share for the noisy component
	 */
	virtual void
	randomize_clustering(soft_clustering_type& _soft_clustering) const {
	  // For each sample
	  for (typename soft_clustering_type::size_type i = 0;
	       i < _soft_clustering.size(); ++i) {
	    // For each cluster
	    value_type sum = this->random_generator_();
	    for (typename soft_clustering_type::size_type cl = 0; 
		 cl < _soft_clustering.clusters(); ++cl) 
	      sum += (_soft_clustering[i][cl] = this->random_generator_());
	    
	    // Normalize the sum
	    for (typename soft_clustering_type::size_type cl = 0; 
		 cl < _soft_clustering.clusters(); ++cl) 
	      _soft_clustering[i][cl] /= sum;
	  }
	}
      };
    }
  }
}

#endif
