#ifndef TTCL_IT_HARD_ITERATIVE_MODEL_HXX
#define TTCL_IT_HARD_ITERATIVE_MODEL_HXX

// TTCL: The Template Clustering Library

/** @file
    Iterative Algorithms - Hard Iterative Clusterer
    @author Edgar Gonzalez i Pellicer
*/

#include <boost/tuple/tuple.hpp>

#include <ttcl/types.hxx>
#include <ttcl/cl/choices.hxx>
#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/it/choices.hxx>
#include <ttcl/tu/choices.hxx>
#include <ttcl/tu/tuning.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Iterative Clustering Namespace
  namespace it {
    
    /// Hard Iterative Model
    template <typename Data,
	      typename Choices = tu::default_choices>
    class hard_iterative_model {
    public:
      /// Data type
      typedef Data data_type;

      /// Hard clustering container
      typedef ttcl_tu_get_choice(Choices, cl::hard_clustering_container)
	hard_clustering_container_type;

      /// Hard clustering type
      typedef typename cl::hard_clustering<hard_clustering_container_type>
        hard_clustering_type;

      /// Value type
      typedef ttcl_tu_get_choice(Choices, model_value_type) value_type;

    protected:
      /// Number of clusters
      uint clusters_;

    public:
      /// Empty constructor
      hard_iterative_model() :
	clusters_(0) {
      }

      /// Constructor
      hard_iterative_model(const uint _clusters) :
	clusters_(_clusters) {
      }

      /// Destructor
      virtual ~hard_iterative_model() {
      }

      /// @name Setters/Getters
      /// @{

      /// Number of clusters
      uint clusters() const {
	  return clusters_;
      }
      
      /// Set the number of clusters
      void set_clusters(uint _clusters) {
	clusters_ = _clusters;
	resized();
      }

      /// @}

      /// Refine
      virtual value_type hard_refine(const data_type& _data,
				     hard_clustering_type& _clustering) = 0;

      /// Cluster
      virtual value_type
      hard_cluster(const data_type& _data,
		   hard_clustering_type& _clustering) const = 0;

    protected:
      /// Resized
      virtual void resized() {
	// Nothing
      }
    };
  }
}

#endif
