#ifndef TTCL_LG_ANSI_OSTREAM_LOGGER_HXX
#define TTCL_LG_ANSI_OSTREAM_LOGGER_HXX

// TTCL: The Template Clustering Library

/** @file
    Logging - Ostream Logger Class using ANSI colors
    @author Edgar Gonzlez i Pellicer
*/

#include <unistd.h>

#include <ttcl/lg/ostream_logger.hxx>
#include <ttcl/ut/ansi.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Logging
  namespace lg {

    /// ANSI Ostream logger class
    class ansi_ostream_logger : public ostream_logger {
    public:
      /// Constructor
      ansi_ostream_logger(std::ostream& _ostream = std::clog) :
	ostream_logger(_ostream) {
      }

    protected:
      /// Log
      virtual void raw_log(level _level, const char* _msg) {
	// According to the level
	switch (_level) {
	case ERROR:
	  ostream_ << ut::ansi_fg_red;
	  break;
	case WARN:
	  ostream_ << ut::ansi_fg_yellow;
	  break;
	case SUCCESS:
	  ostream_ << ut::ansi_fg_green;
	  break;
	default:
	  break;
	}

	// Call the parent version
	ostream_logger::raw_log(_level, _msg);
	ostream_ << ut::ansi_reset;
      }

    public:
      /// Set as logger
      static void set() {
	logger::set_logger(isatty(2) ?
			   new ansi_ostream_logger() : new ostream_logger());
      }
    };
  }
}

#endif    
