#ifndef TTCL_LG_LOGGER_HXX
#define TTCL_LG_LOGGER_HXX

// TTCL: The Template Clustering Library

/** @file
    Logging - Logger Base Class
    @author Edgar Gonzlez i Pellicer
*/

#include <cstdarg>
#include <cstdio>

#include <ttcl/global.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Logging
  namespace lg {

    /// Logger Base Class
    class logger {
    public:
      // Levels
      enum level { ERROR, WARN, INFO, SUCCESS };
 
    private:
      /// Instance
      static logger* instance_;

      // Disable copy constructor
      logger(const logger& _other);

      // Disable assignment operator
      logger& operator=(const logger& _other);

    protected:
      /// Empty constructor
      logger() {
      }

    public:
      /// Destructor
      virtual ~logger() {
      }

      /// Get the instance
      static logger& get_logger() {
	if (not instance_)
	  instance_ = new logger();
	return *instance_;
      }

      /// Set the logger
      static void set_logger(logger* _instance) {
	if (instance_ != _instance)
	  delete instance_;
	instance_ = _instance;
      }

      /// Log
      void log(level _level, const char* _format, ...) ttcl_printf_check(3, 4) {
	// Create an argument list
	va_list list;
	va_start(list, _format);
	log(_level, _format, list);
	va_end(list);
      }

      /// Error
      void error(const char* _format, ...) ttcl_printf_check(2, 3) {
	// Create an argument list
	va_list list;
	va_start(list, _format);
	log(ERROR, _format, list);
	va_end(list);
      }

      /// Warning
      void warn(const char* _format, ...) ttcl_printf_check(2, 3) {
	// Create an argument list
	va_list list;
	va_start(list, _format);
	log(WARN, _format, list);
	va_end(list);
      }

      /// Info
      void info(const char* _format, ...) ttcl_printf_check(2, 3) {
	// Create an argument list
	va_list list;
	va_start(list, _format);
	log(INFO, _format, list);
	va_end(list);
      }

      /// Success
      void success(const char* _format, ...) ttcl_printf_check(2, 3) {
	// Create an argument list
	va_list list;
	va_start(list, _format);
	log(SUCCESS, _format, list);
	va_end(list);
      }

    protected:
      /// Log
      void log(level _level, const char* _format, std::va_list _args) {
	// Print on a buffer
	char buffer[1024];
	vsnprintf(buffer, 1024, _format, _args);

	// Log the message
	raw_log(_level, buffer);
      }

      /// Log
      virtual void raw_log(level _level, const char* _msg) {
	// By default, nothing
      }
    };
  }
}

// Static vars
#define ttcl_lg_static\
  ttcl::lg::logger* ttcl::lg::logger::instance_;

// Get the instance
#define ttcl_logger()\
  ttcl::lg::logger::get_logger()

#endif
