#ifndef TTCL_LG_OSTREAM_LOGGER_HXX
#define TTCL_LG_OSTREAM_LOGGER_HXX

// TTCL: The Template Clustering Library

/** @file
    Logging - Ostream Logger Class
    @author Edgar Gonzlez i Pellicer
*/

#include <ctime>
#include <iostream>

#include <ttcl/lg/logger.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Logging
  namespace lg {

    /// Ostream logger class
    class ostream_logger : public logger {
    protected:
      /// Ostream reference
      std::ostream& ostream_;

    public:
      /// Constructor
      ostream_logger(std::ostream& _ostream = std::clog) : ostream_(_ostream) {
      }

    protected:
      /// Log
      virtual void raw_log(level _level, const char* _msg) {
	// Display the message and timestamp
	ostream_ << now() << _msg << std::endl;
      }

      /// Now
      const char* now() {
	static char buffer[1024];
	time_t t = time(0);
	tm* st   = localtime(&t);
	std::strftime(buffer, 1024, "[%d/%m/%Y %H:%M:%S] ", st);
	return buffer;
      }

    public:
      /// Set as logger
      static void set() {
	logger::set_logger(new ostream_logger());
      }
    };
  }
}

#endif
