#ifndef TTCL_TU_CHOICES_HXX
#define TTCL_TU_CHOICES_HXX

// TTCL: The Template Clustering Library

/** @file
    Tuning - Choices
    @author Edgar Gonzlez i Pellicer
*/

#include <ttcl/ut/null_type.hxx>
#include <ttcl/ut/type_cons.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Tuning Namespace
  namespace tu {

    /// Choice
    template <typename T>
    struct choices {};

    
    /// Add a choice
    template <typename Choices,
	      template <typename> class Keyword,
	      typename Type>
    struct add_choice;
    
    /// Add a choice
    /** Specialization
     */
    template <typename Content,
	      template <typename> class Keyword,
	      typename Type>
    struct add_choice<choices<Content>, Keyword, Type> {
      typedef choices< ut::type_cons< Keyword<Type>, Content > > type;
    };


    /// Add a choice which is a template
    template <typename Choices,
	      template <template <typename> class> class Keyword,
	      template <typename> class Template>
    struct add_choice_tem;

    /// Add a choice which is a template
    /** Specialization
     */
    template <typename Content,
	      template <template <typename> class> class Keyword,
	      template <typename> class Template>
    struct add_choice_tem<choices<Content>, Keyword, Template> {
      typedef choices< ut::type_cons< Keyword<Template>, Content > > type;
    };


    /// Find a type within a choice
    template <typename Choices,
	      template <typename> class Keyword>
    struct type_in_choices;
    
    /// Find a type within a choice
    /** Specialization for the empty list
     */
    template <template <typename> class Keyword>
    struct type_in_choices< choices< ut::null_type >,
			    Keyword > {
      typedef typename Keyword< ut::null_type >::default_type type;
    };

    /// Find a type within a choice
    /** Specialization for a match
     */
    template <template <typename> class Keyword,
	      typename Tail,
	      typename Type>
    struct type_in_choices< choices< ut::type_cons< Keyword<Type>, Tail > >,
			    Keyword > {
      typedef Type type;
    };

    /// Find a type within a choice
    /** Specialization for a skip
     */
    template <typename Head,
	      typename Tail,
	      template <typename> class Keyword>
    struct type_in_choices< choices< ut::type_cons< Head, Tail > >,
			    Keyword > {
      typedef typename type_in_choices< choices< Tail >, Keyword >::type type;
    };


    /// Instantiate a template within a choice
    template <typename Choices,
	      template <template <typename> class> class Keyword,
	      typename Type>
    struct type_in_choices_tem;

    /// Instantiate a template within a choice
    /** Specialization for the empty list
     */
    template <template <template <typename> class> class Keyword,
	      typename Type>
    struct type_in_choices_tem< choices< ut::null_type >,
				Keyword, Type> {
      typedef typename Keyword< ut::null_template >::template default_tem<Type>::type type;
    };

    /// Instantiate a template within a choice
    /** Specialization for a match
     */
    template <template <template <typename> class> class Keyword,
	      typename Tail,
	      template <typename> class Template,
	      typename Type>
    struct type_in_choices_tem< choices< ut::type_cons< Keyword<Template>, Tail > >,
				Keyword, Type > {
      typedef Template<Type> type;
    };

    /// Instantiate a template within a choice
    /** Specialization for a skip
     */
    template <typename Head,
	      typename Tail,
	      template <template <typename> class> class Keyword,
	      typename Type>
    struct type_in_choices_tem< choices< ut::type_cons< Head, Tail > >,
				Keyword, Type > {
      typedef typename type_in_choices_tem< choices< Tail >,
					    Keyword, Type >::type type;
    };


    /// Default choices
    typedef choices< ut::null_type > default_choices;
  }
}

/// Define a choice
#define ttcl_tu_choice(name, deflt_type)		\
  template <typename Type> struct name {};		\
  template <> struct name< ttcl::ut::null_type >  {	\
    typedef deflt_type default_type;			\
  };

/// Define a template choice
#define ttcl_tu_choice_tem(name, deflt_tem)			\
  template <template <typename> class Tem> struct name {};	\
  template <> struct name< ttcl::ut::null_template >  {		\
    template <typename T> struct default_tem {			\
      typedef deflt_tem<T> type;				\
    };								\
  };

/// Add a choice
#define ttcl_tu_add_choice(choices, name, typ)		\
  ttcl::tu::add_choice<choices, name, typ>::type

/// Add a template choice
#define ttcl_tu_add_choice_tem(choices, name, tem)	\
  ttcl::tu::add_choice_tem<choices, name, tem>::type

// Get a type
#define ttcl_tu_get_choice(choices, name)			\
  typename ttcl::tu::type_in_choices<choices, name>::type

// Instantiate a type
#define ttcl_tu_get_choice_tem(choices, name, typ)			\
  typename ttcl::tu::type_in_choices_tem<choices, name, typ>::type

#endif
