#ifndef TTCL_UT_FILLABLE_HXX
#define TTCL_UT_FILLABLE_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - Fillable base class
    @author Edgar Gonzalez i Pellicer
*/

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 

    /// Fillable base class
    /** Uses the Curiously Recurring Template Pattern
     */
    template <typename Child>
    class fillable {
    public:
      /// Filler proxy class
      class filler_proxy {
      private:
	/// Target
	Child& target_;

	/// Constructor
	filler_proxy(Child& _target) :
	  target_(_target) {
	}

      public:
	/// One-parameter fill
	template <typename Value>
	filler_proxy& operator()(const Value& _value) {
	  target_.fill(_value);
	  return *this;
	}

	/// Two-parameter fill
	template <typename Key, typename Value>
	filler_proxy& operator()(const Key& _key,
				 const Value& _value) {
	  target_.fill(_key, _value);
	  return *this;
	}

	// Friendship
	friend class fillable;
      };

      /// Filler
      filler_proxy filler() {
	return filler_proxy(*(static_cast<Child*>(this)));
      }
    };
  }
}
      
#endif
