#ifndef TTCL_UT_INDEXER_HXX
#define TTCL_UT_INDEXER_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - Indexer
    @author Edgar Gonzalez i Pellicer
*/

#include <map>
#include <utility>
#include <vector>

#include <ttcl/exception.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 

    /// Indexer
    /** A structure for indexing
     */
    template <typename Key>
    class indexer {
    public:
      /// Key type
      typedef Key key_type;

      /// Data type
      typedef typename std::vector<key_type>::size_type data_type;

      /// Size type
      typedef typename std::vector<key_type>::size_type size_type;

    protected:
      /// The vector of elements
      std::vector<key_type> vect_;
      
      /// The map of elements to integers
      std::map<key_type, data_type> map_;

    public:
      /// Clear
      void clear() {
	vect_.clear();
	map_.clear();
      }

      /// Index
      /** From element to integer (Const)
       */
      data_type operator[](const key_type& _element) const
	throw (exception) {
	// Look for it
	typename std::map<key_type, data_type>::const_iterator it =
	  map_.find(_element);
	if (it != map_.end()) return it->second;
      
	// Throw an exception
	ttcl_fire("Element not indexed");
      }

      /// Index
      /** From element to integer (Non-const)
       */
      data_type operator[](const key_type& _element) {
	// Look for it
	typename std::map<key_type, data_type>::const_iterator it =
	  map_.find(_element);
	if (it != map_.end()) return it->second;
      
	// Add it
	map_.insert(std::make_pair(_element, vect_.size()));
	vect_.push_back(_element);
	return vect_.size() - 1;
      }

      /// Is indexed?
      bool is_indexed(const key_type& _element) const {
	// Look for it
	return map_.find(_element) != map_.end();
      }

      /// Access
      /** From integer to element
       */
      key_type operator[](data_type _index) const
	throw (exception) {
	// Check bound
	if (_index >= vect_.size())
	  ttcl_fire("Index out of bounds");
      
	// Return the element
	return vect_[_index];
      }

      /// Get the size
      size_type size() const {
	return vect_.size();
      }
    };
  }
}

#endif
