#ifndef TTCL_UT_MASK_HXX
#define TTCL_UT_MASK_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - Type mask
    @author Edgar Gonzalez i Pellicer
*/

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 
  
    /// Mask
    template <typename T, typename Real>
    class mask {
    public:
      /// Value type
      typedef T value_type;

      /// Real type
      typedef Real real_type;

    private:
      /// Value
      value_type value_;

      /// Source
      real_type& source_;

      /// Transferrable
      bool transferrable_;

    public:
      /// Constructor
      mask(real_type& _source) :
	value_(value_type(_source)), source_(_source), transferrable_(true) {
      }

    private:
      /// Copy constructor
      /** Disabled
       */
      mask(const mask& _other);

    public:
      /// Destructor
      ~mask() {
	// Transfer unless doomed
	if (transferrable_)
	  source_ = real_type(value_);
      }

      /// Assignment operator
      mask& operator=(const mask& _other) {
	// Just assign the value
	value_ = _other.value_;

	// Return this
	return *this;
      }

      /// Assignment from value
      mask& operator=(const value_type& _value) {
	// Just assign the value
	value_ = _value;

	// Return this
	return *this;
      }

      /// Reset
      void reset() {
	value_         = source_;
	transferrable_ = true;
      }

      /// Ignore
      void ignore() {
	transferrable_ = false;
      }

      /// Address operator
      value_type* operator&() {
	return &value_;
      }

      /// Address operator
      const value_type* operator&() const {
	return &value_;
      }

      /// Conversion to value
      operator value_type& () {
	return value_;
      }

      /// Conversion to value
      operator const value_type& () const {
	return value_;
      }
    };
  }
}

#endif
