#ifndef TTCL_UT_NULL_TYPE_HXX
#define TTCL_UT_NULL_TYPE_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - Null type
    @author Edgar Gonzlez i Pellicer
*/

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut {

    /// Null type
    struct null_type {
      /// Default constructor
      null_type() {}

      /// Argument constructor
      template <typename T>
      null_type(const T&) {}

      /// Copy constructor
      null_type(const null_type&) {}

      /// Assignment operator
      null_type& operator=(const null_type&) {
	// Return this
	return *this;
      }

      /// Assignment operator
      template <typename T>
      null_type& operator=(const T&) {
	// Return this
	return *this;
      }
    };

    /// Sum
    null_type operator+(const null_type&, const null_type&) {
      // Just return null
      return null_type();
    }

    /// Difference
    null_type operator-(const null_type&, const null_type&) {
      // Just return null
      return null_type();
    }

    /// Product
    null_type operator*(const null_type&, const null_type&) {
      // Just return null
      return null_type();
    }

    /// Division
    null_type operator/(const null_type&, const null_type&) {
      // Just return null
      return null_type();
    }

    /// Equality
    bool operator==(const null_type&, const null_type&) {
      // Always true
      return true;
    }

    /// Inequality
    bool operator!=(const null_type&, const null_type&) {
      // Always false
      return false;
    }

    /// Less
    bool operator<(const null_type&, const null_type&) {
      // Always false
      return false;
    }

    /// Greater
    bool operator>(const null_type&, const null_type&) {
      // Always false
      return false;
    }

    /// Less or equal
    bool operator<=(const null_type&, const null_type&) {
      // Always true
      return true;
    }

    /// Greater or equal
    bool operator>=(const null_type&, const null_type&) {
      // Always true
      return true;
    }

    /// Null Template
    template <typename>
    class null_template {
    };
  }
}

#endif
