#ifndef TTCL_UT_PRINTF_BUFFERS_HXX
#define TTCL_UT_PRINTF_BUFFERS_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - printf() buffers
    @author Edgar Gonzalez i Pellicer
*/

#include <cstdarg>
#include <iostream>
#include <sstream>

#include <ttcl/global.hxx>

// Number of buffers
#ifndef TTCL_UT_PRINTF_BUFFERS_NUMBER
#define TTCL_UT_PRINTF_BUFFERS_NUMBER 8
#endif

// Buffer size
#ifndef TTCL_UT_PRINTF_BUFFERS_SIZE
#define TTCL_UT_PRINTF_BUFFERS_SIZE 1024
#endif

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 

    /// printf() buffers
    /** Singleton class
     */
    class printf_buffers {
    private:
      /// Instance
      static printf_buffers* instance_;

      // Buffers
      char buffers_[TTCL_UT_PRINTF_BUFFERS_NUMBER][TTCL_UT_PRINTF_BUFFERS_SIZE];

      // Current buffer
      uint current_;

      // Disable copy constructor
      printf_buffers(const printf_buffers& _other);

      // Disable assignment operator
      printf_buffers& operator=(const printf_buffers& _other);

    protected:
      /// Empty constructor
      printf_buffers() : current_(0) {
      }

    public:
      /// printf()
      const char* printf(const char* _format, ...)  ttcl_printf_check(2, 3) {
	// Create an argument list
	va_list list;
	va_start(list, _format);

	// Print
	vsnprintf(buffers_[current_], TTCL_UT_PRINTF_BUFFERS_SIZE,
		  _format, list);

	// Destroy argument list
	va_end(list);

	// Next buffer and return
	const char* ret = buffers_[current_++];
	current_ %= TTCL_UT_PRINTF_BUFFERS_NUMBER;
	return ret;
      }

      /// Stringify
      template <typename T>
      const char* str(const T& _value) {
	// Dump to a string stream
	std::ostringstream oss;
	oss << _value;

	// Print
	return printf("%s", oss.str().c_str());
      }

      /// Get the buffers
      static printf_buffers& get_buffers() {
	if (not instance_)
	  instance_ = new printf_buffers();
	return *instance_;
      }
    };
  }
}

// Static vars
#define ttcl_ut_printf_static					\
  ttcl::ut::printf_buffers* ttcl::ut::printf_buffers::instance_

// Macros
#define ttcl_p(...)						\
  ttcl::ut::printf_buffers::get_buffers().printf(__VA_ARGS__)
#define ttcl_d(_value)						\
  ttcl::ut::printf_buffers::get_buffers().printf("%d", _value)
#define ttcl_g(_value)						\
  ttcl::ut::printf_buffers::get_buffers().printf("%g", _value)
#define ttcl_str(_value)					\
  ttcl::ut::printf_buffers::get_buffers().str(_value)

#endif
