#ifndef TTCL_UT_STD_RANDOM_HXX
#define TTCL_UT_STD_RANDOM_HXX

// TTCL: The Template Clustering Library

/** @file
    Utility - Standard Random Number Generator
    @author Edgar Gonzalez i Pellicer
*/

#include <ctime>
#include <cstdlib>

#include <ttcl/tu/choices.hxx>

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 

    /// Standard Random Generator Base Class
    class _std_random {
    public:
      /// Seed type
      typedef unsigned int seed_type;

      /// Set the seed
      /** Using the time
       */
      static void
      set_seed() {
	std::srand(std::time(0));
      }

      /// Set the seed
      /** Using a certain value
       */
      static void
      set_seed(const seed_type _seed) {
	std::srand(_seed);
      }
    };

    /// Standard Random Generator
    /** A model of STL Adaptable Generator, Adaptable Unary Function
	and Random Number Generator
     */
    template <typename T>
    class std_random :
      public _std_random {
    public:
      /// Result type
      typedef T result_type;

      /// Argument type
      typedef T argument_type;

      /// Generate a value
      /** Between 0 and 1
       */
      result_type
      operator()() {
	return result_type(std::rand() / (RAND_MAX + 1.0));
      }

      /// Generate a value
      /** Between 0 and max
       */
      result_type
      operator()(const argument_type _max) {
	return result_type(double(_max) * std::rand() / (RAND_MAX + 1.0));
      }
    };
  }
}

#endif
