#ifndef TTCL_UT_TRAITS_HXX
#define TTCL_UT_TRAITS_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - Traits
    @author Edgar Gonzalez i Pellicer
*/

#include <boost/type_traits/integral_constant.hpp>

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 

    /// Voidify a type
    template <typename T>
    struct voidify {
      /// Inner void type
      typedef void type;
    };
  }
}

/// Has type macro
#define ttcl_has_type(trait_name, target_type)			\
  template <typename T, typename Has = typename T::target_type>	\
  struct _ ## trait_name {					\
    typedef void type;						\
  };								\
  template <typename T, typename Has = void>			\
  struct trait_name : public boost::false_type {		\
  };								\
  template <typename T>						\
  struct trait_name<T, typename _ ## trait_name<T>::type> :	\
    public boost::true_type {					\
  }

/// Has field macro
#define ttcl_has_field(trait_name, field)			\
  template <typename T,						\
	    typename Has = void>				\
  struct trait_name : public boost::false_type {		\
  };								\
  template <typename T>						\
  struct trait_name						\
    <T, typename ttcl::ut::voidify<typeof(&T::field)>::type> :	\
    public boost::true_type {					\
  }

#endif
