#ifndef TTCL_UT_TRANSPARENT_PTR_HXX
#define TTCL_UT_TRANSPARENT_PTR_HXX

// TTCL: The Template Clustering Library

/** @file
    Utilities - Transparent Pointer
    @author Edgar Gonzalez i Pellicer
*/

#include <cstdlib>

/// TTCL Namespace
namespace ttcl {

  /// Utility Namespace
  namespace ut { 

    /// Transparent Pointer
    template <typename T>
    class transparent_ptr {
    public:
      /// Element type
      typedef T element_type;

    private:
      /// Pointer
      element_type* pointer_;

    public:
      /// Constructor
      transparent_ptr(element_type* _pointer = 0) :
	pointer_(_pointer) {
      }

      /// Constructor
      transparent_ptr(element_type& _reference) :
	pointer_(&_reference) {
      }

      /// Copy constructor
      transparent_ptr(const transparent_ptr& _other) :
	pointer_(_other.pointer_) {
      }

      /// Assignment operator
      transparent_ptr& operator=(element_type* _pointer) {
	pointer_ = _pointer;
	return *this;
      }

      /// Assignment operator
      transparent_ptr& operator=(const transparent_ptr& _other) {
	pointer_ = _other.pointer_;
	return *this;
      }

      /// Dereference
      element_type& operator*() const throw() {
	// Access
	return *pointer_;
      }

      /// Dereference
      element_type* operator->() const throw() {
	// Access
	return pointer_;
      }

      /// Equality comparison
      bool operator==(const transparent_ptr& _other) {
	// Forward
	return *pointer_ == *_other.pointer_;
      }

      /// Inequality comparison
      bool operator!=(const transparent_ptr& _other) {
	// Forward
	return *pointer_ != *_other.pointer_;
      }

      /// Less than comparison
      bool operator<(const transparent_ptr& _other) {
	// Forward
	return *pointer_ < *_other.pointer_;
      }

      /// Less than or equal comparison
      bool operator<=(const transparent_ptr& _other) {
	// Forward
	return *pointer_ <= *_other.pointer_;
      }

      /// Greater than comparison
      bool operator>(const transparent_ptr& _other) {
	// Forward
	return *pointer_ > *_other.pointer_;
      }

      /// Greater than or equal comparison
      bool operator>=(const transparent_ptr& _other) {
	// Forward
	return *pointer_ >= *_other.pointer_;
      }
    };
  }
}

#endif
