#include <iostream>
#include <string>

#include <ttcl/types.hxx>
#include <ttcl/io/anystream.hxx>

// Main
int main(int argc, const char* argv[]) {
  // Open the first file
  if (argc < 2) return -1;
  std::istream* is = ttcl::io::ianystream::open(argv[1]);

  // Read loop
  char cmd;
  std::cout << "[c:char, d:double, i:int, l:line, s:string, u:uint]? "
	    << std::flush;
  while (std::cin >> cmd) {
    // According to the command
    switch (cmd) {
    case 'c':
    case 'C': {
      // Read a char
      char c;
      *is >> c;
      if (is->good())
	std::cout << " <char> " << c << std::endl;
      break;
    }

    case 'd':
    case 'D': {
      // Read a double
      double d;
      *is >> d;
      if (is->good())
	std::cout << " <double> " << d << std::endl;
      break;
    }

    case 'i':
    case 'I': {
      // Read an integer
      int i;
      *is >> i;
      if (is->good())
	std::cout << " <int> " << i << std::endl;
      break;
    }

    case 'l':
    case 'L': {
      // Read a line
      std::string l;
      std::getline(*is, l);
      if (is->good())
	std::cout << " <line> " << l << std::endl;
      break;
    }

    case 's':
    case 'S': {
      // Read a string
      std::string s;
      *is >> s;
      if (is->good())
	std::cout << " <string> " << s << std::endl;
      break;
    }

    case 'u':
    case 'U': {
      // Read an uint
      ttcl::uint u;
      *is >> u;
      if (is->good())
	std::cout << " <uint> " << u << std::endl;
      break;
    }

    default:
      // No such command
      std::cout << " <no such command>" << std::endl;
    }

    // Error indications
    if (not is->good()) {
      if (is->bad())
	std::cout << " <bad>" << std::endl;
      else if (is->eof())
	std::cout << " <eof>" << std::endl;
      else // is->fail()
	std::cout << " <fail>" << std::endl;
      is->clear();
    }

    // Prompt
    std::cout << "[c:char, d:double, i:int, l:line, s:string, u:uint]? "
	      << std::flush;
  }

  // Close and quit
  delete is;
  return 0;
}
