#include <cfloat>
#include <climits>
#include <cmath>
#include <sstream>

#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>

#include <ttcl/io/ibinstream.hxx>
#include <ttcl/io/obinstream.hxx>

// Temporary file name
const char* temporary_name = "/tmp/binfstream.test.dat";

// Values

// Bool
const bool bool_true  = true;
const bool bool_false = false;

// Char
const char char_null  = '\0';
const char char_space = ' ';
const char char_tab   = '\t';
const char char_one   = '1';
const char char_max   = CHAR_MAX;

// Short
const short short_min  = SHRT_MIN;
const short short_zero = 0;
const short short_one  = 1;
const short short_max  = SHRT_MAX;

// Unsigned short
const unsigned short ushort_zero = 0u;
const unsigned short ushort_one  = 1u;
const unsigned short ushort_max  = USHRT_MAX;

// Int
const int int_min  = INT_MIN;
const int int_zero = 0;
const int int_one  = 1;
const int int_max  = INT_MAX;

// Unsigned int
const unsigned int uint_zero = 0u;
const unsigned int uint_one  = 1u;
const unsigned int uint_max  = UINT_MAX;

// Long
const long long_min  = LONG_MIN;
const long long_zero = 0l;
const long long_one  = 1l;
const long long_max  = LONG_MAX;

// Unsigned long
const unsigned long ulong_zero = 0ul;
const unsigned long ulong_one  = 1ul;
const unsigned long ulong_max  = ULONG_MAX;

// Long long
const long long llong_min  = LLONG_MIN;
const long long llong_zero = 0ll;
const long long llong_one  = 1ll;
const long long llong_max  = LLONG_MAX;

// Unsigned long long
const unsigned long long ullong_zero = 0ull;
const unsigned long long ullong_one  = 1ull;
const unsigned long long ullong_max  = ULLONG_MAX;

// Float
const float float_min  = FLT_MIN;
const float float_zero = 0.0f;
const float float_one  = 1.0f;
const float float_max  = FLT_MAX;
const float float_nan  = NAN;
const float float_huge = HUGE_VALF;
const float float_inf  = INFINITY;

// Double
const double double_min  = DBL_MIN;
const double double_zero = 0.0;
const double double_one  = 1.0;
const double double_max  = DBL_MAX;
const double double_nan  = NAN;
const double double_huge = HUGE_VAL;
const double double_inf  = INFINITY;

// Long double
const long double ldouble_min  = LDBL_MIN;
const long double ldouble_zero = 0.0l;
const long double ldouble_one  = 1.0l;
const long double ldouble_max  = LDBL_MAX;
const long double ldouble_nan  = NAN;
const long double ldouble_huge = HUGE_VALL;
const long double ldouble_inf  = INFINITY;

// String
const char* string_empty    = "";
const char* string_simple   = "string";
const char* string_compound = "string string";

// Do the serialization
void do_serialize(ttcl::io::obinstream& obs) {
  obs << bool_true;  BOOST_REQUIRE(obs.good());
  obs << bool_false; BOOST_REQUIRE(obs.good());

  obs << char_null;  BOOST_REQUIRE(obs.good());
  obs << char_space; BOOST_REQUIRE(obs.good());
  obs << char_tab;   BOOST_REQUIRE(obs.good());
  obs << char_one;   BOOST_REQUIRE(obs.good());
  obs << char_max;   BOOST_REQUIRE(obs.good());

  obs << short_min;  BOOST_REQUIRE(obs.good());
  obs << short_zero; BOOST_REQUIRE(obs.good());
  obs << short_one;  BOOST_REQUIRE(obs.good());
  obs << short_max;  BOOST_REQUIRE(obs.good());

  obs << ushort_zero; BOOST_REQUIRE(obs.good());
  obs << ushort_one;  BOOST_REQUIRE(obs.good());
  obs << ushort_max;  BOOST_REQUIRE(obs.good());

  obs << int_min;  BOOST_REQUIRE(obs.good());
  obs << int_zero; BOOST_REQUIRE(obs.good());
  obs << int_one;  BOOST_REQUIRE(obs.good());
  obs << int_max;  BOOST_REQUIRE(obs.good());

  obs << uint_zero; BOOST_REQUIRE(obs.good());
  obs << uint_one;  BOOST_REQUIRE(obs.good());
  obs << uint_max;  BOOST_REQUIRE(obs.good());

  obs << long_min;  BOOST_REQUIRE(obs.good());
  obs << long_zero; BOOST_REQUIRE(obs.good());
  obs << long_one;  BOOST_REQUIRE(obs.good());
  obs << long_max;  BOOST_REQUIRE(obs.good());

  obs << ulong_zero; BOOST_REQUIRE(obs.good());
  obs << ulong_one;  BOOST_REQUIRE(obs.good());
  obs << ulong_max;  BOOST_REQUIRE(obs.good());

  obs << llong_min;  BOOST_REQUIRE(obs.good());
  obs << llong_zero; BOOST_REQUIRE(obs.good());
  obs << llong_one;  BOOST_REQUIRE(obs.good());
  obs << llong_max;  BOOST_REQUIRE(obs.good());

  obs << ullong_zero; BOOST_REQUIRE(obs.good());
  obs << ullong_one;  BOOST_REQUIRE(obs.good());
  obs << ullong_max;  BOOST_REQUIRE(obs.good());

  obs << float_min;  BOOST_REQUIRE(obs.good());
  obs << float_zero; BOOST_REQUIRE(obs.good());
  obs << float_one;  BOOST_REQUIRE(obs.good());
  obs << float_max;  BOOST_REQUIRE(obs.good());
  obs << float_nan;  BOOST_REQUIRE(obs.good());
  obs << float_huge; BOOST_REQUIRE(obs.good());
  obs << float_inf;  BOOST_REQUIRE(obs.good());

  obs << double_min;  BOOST_REQUIRE(obs.good());
  obs << double_zero; BOOST_REQUIRE(obs.good());
  obs << double_one;  BOOST_REQUIRE(obs.good());
  obs << double_max;  BOOST_REQUIRE(obs.good());
  obs << double_nan;  BOOST_REQUIRE(obs.good());
  obs << double_huge; BOOST_REQUIRE(obs.good());
  obs << double_inf;  BOOST_REQUIRE(obs.good());

  obs << ldouble_min;  BOOST_REQUIRE(obs.good());
  obs << ldouble_zero; BOOST_REQUIRE(obs.good());
  obs << ldouble_one;  BOOST_REQUIRE(obs.good());
  obs << ldouble_max;  BOOST_REQUIRE(obs.good());
  obs << ldouble_nan;  BOOST_REQUIRE(obs.good());
  obs << ldouble_huge; BOOST_REQUIRE(obs.good());
  obs << ldouble_inf;  BOOST_REQUIRE(obs.good());

  obs << string_empty;    BOOST_REQUIRE(obs.good());
  obs << string_simple;   BOOST_REQUIRE(obs.good());
  obs << string_compound; BOOST_REQUIRE(obs.good());
}

// Do the deserialization
void do_deserialize(ttcl::io::ibinstream& ibs) {
  {
    bool bool_value;
    ibs >> bool_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(bool_value, bool_true);
    ibs >> bool_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(bool_value, bool_false);
  }
		
  {
    char char_value;
    ibs >> char_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(char_value, char_null);
    ibs >> char_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(char_value, char_space);
    ibs >> char_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(char_value, char_tab);
    ibs >> char_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(char_value, char_one);
    ibs >> char_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(char_value, char_max);
  }

  {
    short short_value;
    ibs >> short_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(short_value, short_min);
    ibs >> short_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(short_value, short_zero);
    ibs >> short_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(short_value, short_one);
    ibs >> short_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(short_value, short_max);
  }

  {
    unsigned short ushort_value;
    ibs >> ushort_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ushort_value, ushort_zero);
    ibs >> ushort_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ushort_value, ushort_one);
    ibs >> ushort_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ushort_value, ushort_max);
  }

  {
    int int_value;
    ibs >> int_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(int_value, int_min);
    ibs >> int_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(int_value, int_zero);
    ibs >> int_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(int_value, int_one);
    ibs >> int_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(int_value, int_max);
  }

  {
    unsigned int uint_value;
    ibs >> uint_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(uint_value, uint_zero);
    ibs >> uint_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(uint_value, uint_one);
    ibs >> uint_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(uint_value, uint_max);
  }

  {
    long long_value;
    ibs >> long_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(long_value, long_min);
    ibs >> long_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(long_value, long_zero);
    ibs >> long_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(long_value, long_one);
    ibs >> long_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(long_value, long_max);
  }

  {
    unsigned long ulong_value;
    ibs >> ulong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ulong_value, ulong_zero);
    ibs >> ulong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ulong_value, ulong_one);
    ibs >> ulong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ulong_value, ulong_max);
  }

  {
    long long llong_value;
    ibs >> llong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(llong_value, llong_min);
    ibs >> llong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(llong_value, llong_zero);
    ibs >> llong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(llong_value, llong_one);
    ibs >> llong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(llong_value, llong_max);
  }

  {
    unsigned long long ullong_value;
    ibs >> ullong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ullong_value, ullong_zero);
    ibs >> ullong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ullong_value, ullong_one);
    ibs >> ullong_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ullong_value, ullong_max);
  }

  {
    float float_value;
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(float_value, float_min);
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(float_value, float_zero);
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(float_value, float_one);
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(float_value, float_max);
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK(std::isnan(float_value));
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(float_value, float_huge);
    ibs >> float_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(float_value, float_inf);
  }

  {
    double double_value;
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(double_value, double_min);
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(double_value, double_zero);
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(double_value, double_one);
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(double_value, double_max);
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK(std::isnan(double_value));
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(double_value, double_huge);
    ibs >> double_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(double_value, double_inf);
  }

  {
    long double ldouble_value;
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ldouble_value, ldouble_min);
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ldouble_value, ldouble_zero);
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ldouble_value, ldouble_one);
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ldouble_value, ldouble_max);
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK(std::isnan(ldouble_nan));
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ldouble_value, ldouble_huge);
    ibs >> ldouble_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK_EQUAL(ldouble_value, ldouble_inf);
  }

  {
    char string_value[128];
    ibs >> string_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK(not std::strcmp(string_value, string_empty));
    ibs >> string_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK(not std::strcmp(string_value, string_simple));
    ibs >> string_value; BOOST_REQUIRE(ibs.good());
    BOOST_CHECK(not std::strcmp(string_value, string_compound));
  }
}

// Serialize to file
BOOST_AUTO_TEST_CASE( serialize_file ) {
  // Open the output stream
  ttcl::io::obinstream obs;
  obs.open(temporary_name);
  BOOST_REQUIRE(obs.is_open());

  // Do
  do_serialize(obs);
  do_serialize(obs);

  // Close
  obs.close();

  // Open the input stream
  ttcl::io::ibinstream ibs;
  ibs.open(temporary_name);
  BOOST_REQUIRE(ibs.is_open());

  // Do
  do_deserialize(ibs);
  do_deserialize(ibs);

  // Close
  ibs.close();

  // Unlink
  unlink(temporary_name);
}

// Serialize to string
BOOST_AUTO_TEST_CASE( serialize_string ) {
  // Open the output stream
  std::ostringstream oss;
  ttcl::io::obinstream obs(oss);
  BOOST_REQUIRE(obs.is_open());

  // Do
  do_serialize(obs);
  do_serialize(obs);

  // Close
  obs.close();

  // Open the input stream
  std::istringstream iss(oss.str());
  ttcl::io::ibinstream ibs(iss);
  BOOST_REQUIRE(ibs.is_open());

  // Do
  do_deserialize(ibs);
  do_deserialize(ibs);

  // Close
  ibs.close();
}
