#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>

#include <ttcl/types.hxx>
#include <ttcl/co/bit_set.hxx>

// Static
ttcl_co_bit_set_static;

// Number of elements
const ttcl::uint nelems = 16;

// First set
const ttcl::uint set1[] = { 0, 1, 2, 3 };
const ttcl::uint nset1 = 4;

// Second set
const ttcl::uint set2[] = { 1, 2, 4, 5, 6 };
const ttcl::uint nset2 = 5;

// Third set
const ttcl::uint set3[] = { 2, 4, 6, 8, 10, 12, 14, 15 };
const ttcl::uint nset3 = 8;

// Bit set
BOOST_AUTO_TEST_CASE( bit_set ) {
  // First
  ttcl::co::bit_set bset1(nelems);
  for (ttcl::uint i = 0; i < nset1; ++i)
    bset1[set1[i]] = true;

  // Check
  BOOST_CHECK(bset1.max_size() >= set1[nset1 - 1] + 1);
  BOOST_CHECK(not bset1.empty());
  BOOST_CHECK_EQUAL(bset1.size(), nset1);
  ttcl::uint prev = 0;
  for (ttcl::uint i = 0; i < nset1; ++i) {
    for (ttcl::uint j = prev; j < set1[i]; ++j)
      BOOST_CHECK_EQUAL(bset1[j], false);
    BOOST_CHECK_EQUAL(bset1[set1[i]], true);
    prev = set1[i] + 1;
  }


  // Second
  ttcl::co::bit_set bset2(nelems);
  for (ttcl::uint i = 0; i < nset2; ++i)
    bset2[set2[i]] = true;

  // Check
  BOOST_CHECK(bset2.max_size() >= set2[nset2 - 1] + 1);
  BOOST_CHECK(not bset2.empty());
  BOOST_CHECK_EQUAL(bset2.size(), nset2);
  prev = 0;
  for (ttcl::uint i = 0; i < nset2; ++i) {
    for (ttcl::uint j = prev; j < set2[i]; ++j)
      BOOST_CHECK_EQUAL(bset2[j], false);
    BOOST_CHECK_EQUAL(bset2[set2[i]], true);
    prev = set2[i] + 1;
  }

  // Third
  ttcl::co::bit_set bset3(nelems);
  for (ttcl::uint i = 0; i < nset3; ++i)
    bset3[set3[i]] = true;

  // Check
  BOOST_CHECK(bset3.max_size() >= set3[nset3 - 1] + 1);
  BOOST_CHECK(not bset3.empty());
  BOOST_CHECK_EQUAL(bset3.size(), nset3);
  prev = 0;
  for (ttcl::uint i = 0; i < nset3; ++i) {
    for (ttcl::uint j = prev; j < set3[i]; ++j)
      BOOST_CHECK_EQUAL(bset3[j], false);
    BOOST_CHECK_EQUAL(bset3[set3[i]], true);
    prev = set3[i] + 1;
  }


  // Comparison bset1 bset1
  BOOST_CHECK(     bset1 == bset1 );
  BOOST_CHECK(not (bset1 != bset1));
  BOOST_CHECK(not (bset1 <  bset1));
  BOOST_CHECK(     bset1 <= bset1 );
  BOOST_CHECK(not (bset1 >  bset1));
  BOOST_CHECK(     bset1 >= bset1 );

  // Comparison bset2 bset2
  BOOST_CHECK(     bset2 == bset2 );
  BOOST_CHECK(not (bset2 != bset2));
  BOOST_CHECK(not (bset2 <  bset2));
  BOOST_CHECK(     bset2 <= bset2 );
  BOOST_CHECK(not (bset2 >  bset2));
  BOOST_CHECK(     bset2 >= bset2 );

  // Comparison bset3 bset3
  BOOST_CHECK(     bset3 == bset3 );
  BOOST_CHECK(not (bset3 != bset3));
  BOOST_CHECK(not (bset3 <  bset3));
  BOOST_CHECK(     bset3 <= bset3 );
  BOOST_CHECK(not (bset3 >  bset3));
  BOOST_CHECK(     bset3 >= bset3 );

  // Comparison bset1 bset2
  BOOST_CHECK(not (bset1 == bset2));
  BOOST_CHECK(     bset1 != bset2 );
  BOOST_CHECK(     bset1 <  bset2 );
  BOOST_CHECK(     bset1 <= bset2 );
  BOOST_CHECK(not (bset1 >  bset2));
  BOOST_CHECK(not (bset1 >= bset2));

  // Comparison bset2 bset1
  BOOST_CHECK(not (bset2 == bset1));
  BOOST_CHECK(     bset2 != bset1 );
  BOOST_CHECK(not (bset2 <  bset1));
  BOOST_CHECK(not (bset2 <= bset1));
  BOOST_CHECK(     bset2 >  bset1 );
  BOOST_CHECK(     bset2 >= bset1 );

  // Comparison bset1 bset3
  BOOST_CHECK(not (bset1 == bset3));
  BOOST_CHECK(     bset1 != bset3 );
  BOOST_CHECK(     bset1 <  bset3 );
  BOOST_CHECK(     bset1 <= bset3 );
  BOOST_CHECK(not (bset1 >  bset3));
  BOOST_CHECK(not (bset1 >= bset3));

  // Comparison bset3 bset1
  BOOST_CHECK(not (bset3 == bset1));
  BOOST_CHECK(     bset3 != bset1 );
  BOOST_CHECK(not (bset3 <  bset1));
  BOOST_CHECK(not (bset3 <= bset1));
  BOOST_CHECK(     bset3 >  bset1 );
  BOOST_CHECK(     bset3 >= bset1 );

  // Comparison bset2 bset3
  BOOST_CHECK(not (bset2 == bset3));
  BOOST_CHECK(     bset2 != bset3 );
  BOOST_CHECK(     bset2 <  bset3 );
  BOOST_CHECK(     bset2 <= bset3 );
  BOOST_CHECK(not (bset2 >  bset3));
  BOOST_CHECK(not (bset2 >= bset3));

  // Comparison bset3 bset2
  BOOST_CHECK(not (bset3 == bset2));
  BOOST_CHECK(     bset3 != bset2 );
  BOOST_CHECK(not (bset3 <  bset2));
  BOOST_CHECK(not (bset3 <= bset2));
  BOOST_CHECK(     bset3 >  bset2 );
  BOOST_CHECK(     bset3 >= bset2 );


  // Unions
  ttcl::co::bit_set u12 = set_union(bset1, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(u12[i], bset1[i] or bset2[i]);
  ttcl::co::bit_set u21 = set_union(bset2, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(u21[i], bset2[i] or bset1[i]);
  ttcl::co::bit_set u13 = set_union(bset1, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(u13[i], bset1[i] or bset3[i]);
  ttcl::co::bit_set u31 = set_union(bset3, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(u31[i], bset3[i] or bset1[i]);
  ttcl::co::bit_set u23 = set_union(bset2, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(u23[i], bset2[i] or bset3[i]);
  ttcl::co::bit_set u32 = set_union(bset3, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(u32[i], bset3[i] or bset2[i]);


  // Intersections
  ttcl::co::bit_set i12 = set_intersection(bset1, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(i12[i], bset1[i] and bset2[i]);
  ttcl::co::bit_set i21 = set_intersection(bset2, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(i21[i], bset2[i] and bset1[i]);
  ttcl::co::bit_set i13 = set_intersection(bset1, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(i13[i], bset1[i] and bset3[i]);
  ttcl::co::bit_set i31 = set_intersection(bset3, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(i31[i], bset3[i] and bset1[i]);
  ttcl::co::bit_set i23 = set_intersection(bset2, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(i23[i], bset2[i] and bset3[i]);
  ttcl::co::bit_set i32 = set_intersection(bset3, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(i32[i], bset3[i] and bset2[i]);

  // Differences
  ttcl::co::bit_set d12 = set_difference(bset1, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(d12[i], bset1[i] and not bset2[i]);
  ttcl::co::bit_set d21 = set_difference(bset2, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(d21[i], bset2[i] and not bset1[i]);
  ttcl::co::bit_set d13 = set_difference(bset1, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(d13[i], bset1[i] and not bset3[i]);
  ttcl::co::bit_set d31 = set_difference(bset3, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(d31[i], bset3[i] and not bset1[i]);
  ttcl::co::bit_set d23 = set_difference(bset2, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(d23[i], bset2[i] and not bset3[i]);
  ttcl::co::bit_set d32 = set_difference(bset3, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(d32[i], bset3[i] and not bset2[i]);


  // Symmetric Differences
  ttcl::co::bit_set sd12 = set_symmetric_difference(bset1, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(sd12[i], bset1[i] xor bset2[i]);
  ttcl::co::bit_set sd21 = set_symmetric_difference(bset2, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(sd21[i], bset2[i] xor bset1[i]);
  ttcl::co::bit_set sd13 = set_symmetric_difference(bset1, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(sd13[i], bset1[i] xor bset3[i]);
  ttcl::co::bit_set sd31 = set_symmetric_difference(bset3, bset1);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(sd31[i], bset3[i] xor bset1[i]);
  ttcl::co::bit_set sd23 = set_symmetric_difference(bset2, bset3);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(sd23[i], bset2[i] xor bset3[i]);
  ttcl::co::bit_set sd32 = set_symmetric_difference(bset3, bset2);
  for (ttcl::uint i = 0; i < nelems; ++i)
    BOOST_CHECK_EQUAL(sd32[i], bset3[i] xor bset2[i]);

  // First bits
  BOOST_CHECK_EQUAL(bset1.front(), 0);
  BOOST_CHECK_EQUAL(bset2.front(), 1);
  BOOST_CHECK_EQUAL(bset3.front(), 2);

  BOOST_CHECK_EQUAL(u12  .front(), 0);
  BOOST_CHECK_EQUAL(u21  .front(), 0);
  BOOST_CHECK_EQUAL(u13  .front(), 0);
  BOOST_CHECK_EQUAL(u31  .front(), 0);
  BOOST_CHECK_EQUAL(u23  .front(), 1);
  BOOST_CHECK_EQUAL(u32  .front(), 1);

  BOOST_CHECK_EQUAL(i12  .front(), 1);
  BOOST_CHECK_EQUAL(i21  .front(), 1);
  BOOST_CHECK_EQUAL(i13  .front(), 2);
  BOOST_CHECK_EQUAL(i31  .front(), 2);
  BOOST_CHECK_EQUAL(i23  .front(), 2);
  BOOST_CHECK_EQUAL(i32  .front(), 2);

  BOOST_CHECK_EQUAL(d12  .front(), 0);
  BOOST_CHECK_EQUAL(d21  .front(), 4);
  BOOST_CHECK_EQUAL(d13  .front(), 0);
  BOOST_CHECK_EQUAL(d31  .front(), 4);
  BOOST_CHECK_EQUAL(d23  .front(), 1);
  BOOST_CHECK_EQUAL(d32  .front(), 8);

  BOOST_CHECK_EQUAL(sd12 .front(), 0);
  BOOST_CHECK_EQUAL(sd21 .front(), 0);
  BOOST_CHECK_EQUAL(sd13 .front(), 0);
  BOOST_CHECK_EQUAL(sd31 .front(), 0);
  BOOST_CHECK_EQUAL(sd23 .front(), 1);
  BOOST_CHECK_EQUAL(sd32 .front(), 1);

  // Systematic
  for (uint i = 0; i < 255; ++i) {
    ttcl::co::bit_set bs(i + 1);
    bs[i] = true;
    BOOST_CHECK_EQUAL(bs.front(), i);
  }
}
