#include <vector>

#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>

#include <ttcl/tu/choices.hxx>

using ttcl::tu::default_choices;

// Types
struct type_a { char id() { return 'a'; } };
struct type_b { char id() { return 'b'; } };
struct type_c { char id() { return 'c'; } };

// Templates
template <typename T> struct tem_a { char id() { return 'a'; } };
template <typename T> struct tem_b { char id() { return 'b'; } };
template <typename T> struct tem_c { char id() { return 'c'; } };

// Test type choice
ttcl_tu_choice(test_type1, type_a);
ttcl_tu_choice(test_type2, type_c);
ttcl_tu_choice_tem(test_tem1, tem_a);
ttcl_tu_choice_tem(test_tem2, tem_c);

// Choices
typedef ttcl_tu_add_choice(default_choices, test_type1, type_b) choices_bcac;
typedef ttcl_tu_add_choice(default_choices, test_type2, type_b) choices_abac;
typedef ttcl_tu_add_choice(choices_bcac,    test_type1, type_c) choices_ccac;
typedef ttcl_tu_add_choice(choices_abac,    test_type2, type_a) choices_aaac;

typedef ttcl_tu_add_choice_tem(choices_abac, test_tem1, tem_b)  choices_abbc;
typedef ttcl_tu_add_choice_tem(choices_abbc, test_tem1, tem_c)  choices_abcc;
typedef ttcl_tu_add_choice_tem(choices_abcc, test_tem2, tem_b)  choices_abcb;

// Test class
template <typename Choices = default_choices>
struct test {
  typedef ttcl_tu_get_choice(Choices, test_type1) type1;
  typedef ttcl_tu_get_choice(Choices, test_type2) type2;

  typedef ttcl_tu_get_choice_tem(Choices, test_tem1, int) tem1;
  typedef ttcl_tu_get_choice_tem(Choices, test_tem2, int) tem2;

  char id1()  { type1 v; return v.id(); }
  char id2()  { type2 v; return v.id(); }
  char idt1() { tem1  v; return v.id(); }
  char idt2() { tem2  v; return v.id(); }
};

// Test cases
BOOST_AUTO_TEST_CASE( test_case ) {
  test<> t_acac;
  BOOST_CHECK_EQUAL(t_acac.id1(),  'a');
  BOOST_CHECK_EQUAL(t_acac.id2(),  'c');
  BOOST_CHECK_EQUAL(t_acac.idt1(), 'a');
  BOOST_CHECK_EQUAL(t_acac.idt2(), 'c');

  test<choices_bcac> t_bcac;
  BOOST_CHECK_EQUAL(t_bcac.id1(), 'b');
  BOOST_CHECK_EQUAL(t_bcac.id2(), 'c');
  BOOST_CHECK_EQUAL(t_acac.idt1(), 'a');
  BOOST_CHECK_EQUAL(t_acac.idt2(), 'c');
  
  test<choices_abac> t_abac;
  BOOST_CHECK_EQUAL(t_abac.id1(), 'a');
  BOOST_CHECK_EQUAL(t_abac.id2(), 'b');
  BOOST_CHECK_EQUAL(t_acac.idt1(), 'a');
  BOOST_CHECK_EQUAL(t_acac.idt2(), 'c');

  test<choices_ccac> t_ccac;
  BOOST_CHECK_EQUAL(t_ccac.id1(), 'c');
  BOOST_CHECK_EQUAL(t_ccac.id2(), 'c');
  BOOST_CHECK_EQUAL(t_acac.idt1(), 'a');
  BOOST_CHECK_EQUAL(t_acac.idt2(), 'c');

  test<choices_aaac> t_aaac;
  BOOST_CHECK_EQUAL(t_aaac.id1(), 'a');
  BOOST_CHECK_EQUAL(t_aaac.id2(), 'a');
  BOOST_CHECK_EQUAL(t_acac.idt1(), 'a');
  BOOST_CHECK_EQUAL(t_acac.idt2(), 'c');

  test<choices_abbc> t_abbc;
  BOOST_CHECK_EQUAL(t_abbc.id1(), 'a');
  BOOST_CHECK_EQUAL(t_abbc.id2(), 'b');
  BOOST_CHECK_EQUAL(t_abbc.idt1(), 'b');
  BOOST_CHECK_EQUAL(t_abbc.idt2(), 'c');

  test<choices_abcc> t_abcc;
  BOOST_CHECK_EQUAL(t_abcc.id1(), 'a');
  BOOST_CHECK_EQUAL(t_abcc.id2(), 'b');
  BOOST_CHECK_EQUAL(t_abcc.idt1(), 'c');
  BOOST_CHECK_EQUAL(t_abcc.idt2(), 'c');

  test<choices_abcb> t_abcb;
  BOOST_CHECK_EQUAL(t_abcb.id1(), 'a');
  BOOST_CHECK_EQUAL(t_abcb.id2(), 'b');
  BOOST_CHECK_EQUAL(t_abcb.idt1(), 'c');
  BOOST_CHECK_EQUAL(t_abcb.idt2(), 'b');
};
