#include <vector>

#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>

#include <ttcl/cl/hard_clustering.hxx>
#include <ttcl/hj/dendrogram.hxx>

// Number of samples
const int n_samples = 8;

// Merges
const int merges[][3] = { { 0,  1,  8 },
			  { 5,  6,  9 },
			  { 3,  4, 10 },
			  { 9, 10, 11 },
			  { 2, 11, 12 },
			  { 8, 12, 13 },
			  { 7, 13, 14 } };

// First step
const int first_step = 3;

// First clusters
const int first_clusters = 2;

// First merge with 1 cluster
const int first_merge_1[] = { 0, 0, 1, 2, 2, 3, 3, 4 };

// First merge with 6 clusters
const int first_merge_6[] = { 0, 0, 1, 2, 3, 4, 4, 5 };

// Final merge with 3 clusters
const int final_merge_3[] = { 0, 0, 1, 1, 1, 1, 1, 2 };

// Cutting a dendrogram
BOOST_AUTO_TEST_CASE( dendrogram_cut ) {
  typedef ttcl::hj::dendrogram<>      dendrogram;
  typedef ttcl::cl::hard_clustering<> clustering;

  dendrogram dendro(n_samples);
  
  // Starting cut
  clustering cl1 = dendro.cut(1);
  BOOST_REQUIRE_EQUAL(cl1.size(), ttcl::uint(n_samples));
  BOOST_CHECK_EQUAL(cl1.clusters(), n_samples);
  for (int i = 0; i < n_samples; ++i)
    BOOST_CHECK_EQUAL(cl1[i], i);

  // Starting cut
  clustering cl2 = dendro.cut(2 * n_samples);
  BOOST_REQUIRE_EQUAL(cl2.size(), ttcl::uint(n_samples));
  BOOST_CHECK_EQUAL(cl2.clusters(), n_samples);
  for (int i = 0; i < n_samples; ++i)
    BOOST_CHECK_EQUAL(cl2[i], i);
  
  // Partially merge
  for (int i = 0; i < first_step; ++i)
    BOOST_REQUIRE_EQUAL(dendro.merge(merges[i][0], merges[i][1]),
			ttcl::uint(merges[i][2]));

  // Cut
  clustering cl3 = dendro.cut(1);
  BOOST_REQUIRE_EQUAL(cl3.size(), ttcl::uint(n_samples));
  BOOST_CHECK_EQUAL(cl3.clusters(), n_samples - first_step);
  BOOST_CHECK_EQUAL(cl3.reindex_clusters(), 5);
  for (int i = 0; i < n_samples; ++i)
    BOOST_CHECK_EQUAL(cl3[i], first_merge_1[i]);

  // Cut
  clustering cl4 = dendro.cut(6);
  BOOST_REQUIRE_EQUAL(cl4.size(), ttcl::uint(n_samples));
  BOOST_CHECK_EQUAL(cl4.clusters(), 6);
  BOOST_CHECK_EQUAL(cl4.reindex_clusters(), 6);
  for (int i = 0; i < n_samples; ++i)
    BOOST_CHECK_EQUAL(cl4[i], first_merge_6[i]);

  // Finish the merge
  for (int i = first_step; i < n_samples - 1; ++i)
    BOOST_REQUIRE_EQUAL(dendro.merge(merges[i][0], merges[i][1]),
			ttcl::uint(merges[i][2]));
  
  // Cut
  clustering cl5 = dendro.cut(3);
  BOOST_REQUIRE_EQUAL(cl5.size(), ttcl::uint(n_samples));
  BOOST_CHECK_EQUAL(cl5.clusters(), 3);
  BOOST_CHECK_EQUAL(cl5.reindex_clusters(), 3);
  for (int i = 0; i < n_samples; ++i)
    BOOST_CHECK_EQUAL(cl5[i], final_merge_3[i]);
}  
