#include <utility>
#include <vector>

#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <ttcl/co/array.hxx>
#include <ttcl/co/fake_pair.hxx>
#include <ttcl/co/feature_container.hxx>

// Sample 1
const ttcl::uint size1 = 4;
const ttcl::uint sample1[size1] = { 1, 2, 3, 4 };

// Sample 2
const ttcl::uint size2 = 5;
const ttcl::uint sample2[size2] = { 3, 4, 5, 6, 7 };

// Sample 3
const ttcl::uint size3 = 3;
const ttcl::uint sample3[size3] = { 3, 6, 9 };

// Sizes
const ttcl::uint  nsamples          = 3;
const ttcl::uint  sizes  [nsamples] = {   size1,   size2,   size3 };
const ttcl::uint* samples[nsamples] = { sample1, sample2, sample3 };

// Test for dense
const ttcl::uint dense_size = 5;
template <template <typename> class Container,
	  template <typename> class Sample>
struct test_dense {
  static void test() {
    typedef Sample   <ttcl::uint>  sample_type;
    typedef Container<sample_type> container_type;
    typedef ttcl::co::feature_container<container_type> effective_type;

    effective_type data;
    data.resize(nsamples);
    for (ttcl::uint i = 0; i < nsamples; ++i) {
      data[i].resize(sizes[i]);
      for (ttcl::uint j = 0; j < nsamples; ++j)
	data[i][j] = samples[i][j];
    }

    BOOST_CHECK_EQUAL(data.features(), dense_size);
  }
};

// Test for sparse
const ttcl::uint sparse_size = 10;
template <template <typename> class Container,
	  template <typename> class Sample,
	  typename Item>
struct test_sparse {
  static void test() {
    typedef Item                   item_type;
    typedef Sample   <item_type>   sample_type;
    typedef Container<sample_type> container_type;
    typedef ttcl::co::feature_container<container_type> effective_type;

    effective_type data;
    data.resize(nsamples);
    for (ttcl::uint i = 0; i < nsamples; ++i) {
      data[i].resize(sizes[i]);
      for (ttcl::uint j = 0; j < nsamples; ++j)
	data[i][j].first = samples[i][j];
    }

    BOOST_CHECK_EQUAL(data.features(), sparse_size);
  }
};

// Partial specializations
template <typename T>
struct partial_vector : public std::vector<T> {};
template <typename T>
struct partial_array: public ttcl::co::array<T> {};
template <typename T>
struct partial_std_pair : public std::pair<T, int> {};
template <typename T>
struct partial_fake_pair : public ttcl::co::fake_pair_second<T, int, 1> {};

// Tests
BOOST_AUTO_TEST_CASE( tests ) {
  // Dense tests
  test_dense<partial_vector, partial_vector>::test();
  test_dense<partial_vector, partial_array> ::test();
  test_dense<partial_array,  partial_vector>::test();
  test_dense<partial_array,  partial_array> ::test();

  // Sparse tests
  typedef std::pair<ttcl::uint, ttcl::uint> partial_std_pair;
  typedef ttcl::co::fake_pair_second<ttcl::uint, ttcl::uint, 1>
    partial_fake_pair;
  test_sparse<partial_vector, partial_vector, partial_std_pair> ::test();
  test_sparse<partial_vector, partial_vector, partial_fake_pair>::test();
  test_sparse<partial_vector, partial_array,  partial_std_pair> ::test();
  test_sparse<partial_vector, partial_array,  partial_fake_pair>::test();
  test_sparse<partial_array,  partial_vector, partial_std_pair> ::test();
  test_sparse<partial_array,  partial_vector, partial_fake_pair>::test();
  test_sparse<partial_array,  partial_array,  partial_std_pair> ::test();
  test_sparse<partial_array,  partial_array,  partial_fake_pair>::test();
}
