#include <algorithm>
#include <functional>
#include <vector>

#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>

#include <ttcl/co/limit_priority_queue.hxx>

// Big max heap
BOOST_AUTO_TEST_CASE( big_max_heap ) {
  ttcl::co::limit_priority_queue<int> Q(1 << 4);
  Q.push(1);
  Q.push(4);
  Q.push(2);
  Q.push(8);
  Q.push(5);
  Q.push(7);
  
  BOOST_CHECK_EQUAL(Q.top(), 8);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 7);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 5);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 4);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 2);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 1);
  Q.pop();
  BOOST_CHECK(Q.empty());
}

// Big min heap
BOOST_AUTO_TEST_CASE( big_min_heap ) {
  ttcl::co::limit_priority_queue<int, std::vector<int>, std::greater<int> > Q(1 << 4);
  Q.push(1);
  Q.push(4);
  Q.push(2);
  Q.push(8);
  Q.push(5);
  Q.push(7);
  
  BOOST_CHECK_EQUAL(Q.top(), 1);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 2);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 4);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 5);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 7);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 8);
  Q.pop();
  BOOST_CHECK(Q.empty());
}

// Small max heap
BOOST_AUTO_TEST_CASE( small_max_heap ) {
  ttcl::co::limit_priority_queue<int> Q(1 << 2);
  Q.push(1);
  Q.push(4);
  Q.push(2);
  Q.push(8);
  Q.push(5);
  Q.push(7);
  
  BOOST_CHECK_EQUAL(Q.top(), 8);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 7);
  Q.pop();
  BOOST_CHECK(Q.empty());
}

// Small min heap
BOOST_AUTO_TEST_CASE( small_min_heap ) {
  ttcl::co::limit_priority_queue<int, std::vector<int>, std::greater<int> > Q(1 << 2);
  Q.push(1);
  Q.push(4);
  Q.push(2);
  Q.push(8);
  Q.push(5);
  Q.push(7);
  
  BOOST_CHECK_EQUAL(Q.top(), 1);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 2);
  Q.pop();
  BOOST_REQUIRE(not Q.empty());
  BOOST_CHECK_EQUAL(Q.top(), 4);
  Q.pop();
  BOOST_CHECK(Q.empty());
}

// Massive max heap
BOOST_AUTO_TEST_CASE( massive_max_heap ) {
  ttcl::co::limit_priority_queue<int> Q(1 <<3);
  std::vector<int> contents;
  
  // From 100 down
  int range = 100;
  while (range > 0) {
    // Generate the numbers to be inserted
    contents.clear();
    for (int i = 0; i < range; ++i)
      contents.push_back(i);
    random_shuffle(contents.begin(), contents.end());

    // Insert them
    Q.clear();
    for (int i = 0; i < range; ++i)
      Q.push(contents[i]);

    // Check not empty
    BOOST_REQUIRE(not Q.empty());
    
    // Extract until it is empty again
    while (not Q.empty()) {
      BOOST_CHECK_EQUAL(Q.top(), --range);
      Q.pop();
    }
  }

  // Check empty
  BOOST_CHECK(Q.empty());
}

// Massive min heap
BOOST_AUTO_TEST_CASE( massive_min_heap ) {
  ttcl::co::limit_priority_queue<int, std::vector<int>, std::greater<int> > Q(1 << 3);
  std::vector<int> contents;
  
  // From 0 up
  int range = 0;
  while (range < 100) {
    // Generate the numbers to be inserted
    contents.clear();
    for (int i = range; i < 100; ++i)
      contents.push_back(i);
    random_shuffle(contents.begin(), contents.end());

    // Insert them
    Q.clear();
    for (int i = range; i < 100; ++i)
      Q.push(contents[i - range]);

    // Check not empty
    BOOST_REQUIRE(not Q.empty());
    
    // Extract until it is empty again
    while (not Q.empty()) {
      BOOST_CHECK_EQUAL(Q.top(), range++);
      Q.pop();
    }
  }

  // Check empty
  BOOST_CHECK(Q.empty());
}
