#include <vector>

#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>

#include <ttcl/tu/tuning.hxx>

// Options
ttcl_tu_option(double, em_threshold, 1.0);
ttcl_tu_option(int, n_fields, 10);

// Static vars
ttcl_tu_static;
ttcl_tu_option_static(em_threshold);
ttcl_tu_option_static(n_fields);

// Tuning
BOOST_AUTO_TEST_CASE( test_tuning ) {
  // Try the default tuning
  ttcl::tu::tuning t1;
  BOOST_CHECK_EQUAL( t1.get<em_threshold>(), 1.0 );
  BOOST_CHECK_EQUAL( t1.get<n_fields>(),     10 );

  // Add an option
  t1.set(new em_threshold(2.3));
  BOOST_CHECK_EQUAL( t1.get<em_threshold>(), 2.3 );
  BOOST_CHECK_EQUAL( t1.get<n_fields>(),     10 );
  
  // Check default is still the same
  ttcl::tu::tuning t2;
  BOOST_CHECK_EQUAL( t2.get<em_threshold>(), 1.0 );
  BOOST_CHECK_EQUAL( t2.get<n_fields>(),     10 );

  // Modify the option
  ttcl::tu::tuning t3 = t1;
  t3.set(new em_threshold(4.5));
  BOOST_CHECK_EQUAL( t1.get<em_threshold>(), 2.3 );
  BOOST_CHECK_EQUAL( t3.get<em_threshold>(), 4.5 );
  BOOST_CHECK_EQUAL( t1.get<n_fields>(),     10 );
  BOOST_CHECK_EQUAL( t3.get<n_fields>(),     10 );

  // Another one
  t3.set(new n_fields(4));
  int nf3 = t3.get<n_fields>();
  BOOST_CHECK_EQUAL( nf3, 4 );

  // Remove
  ttcl::tu::tuning t4 = t3;
  t4.remove<n_fields>();
  BOOST_CHECK_EQUAL( t4.get<em_threshold>(), 4.5 );
  BOOST_CHECK_EQUAL( t4.get<n_fields>(),     10 );
}

  

  
