#define BOOST_TEST_MAIN
#include <boost/test/included/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <ttcl/ut/traits.hxx>

// Traits
ttcl_has_field(has_norm, norm);
ttcl_has_field(has_value, value);
ttcl_has_type(has_norm_type, norm);
ttcl_has_type(has_value_type, value);

// Classes
struct A {
  typedef double norm;
  typedef void value;
};

struct B {
  int value;
  int norm() const { return 2; }
};

struct C {
  typedef void value;
  int norm() const { return 3; }
};

struct D {
};

struct E {
  int norm;
};

// Has norm
BOOST_AUTO_TEST_CASE( test_has_norm ) {
  BOOST_CHECK(not has_norm<A>::value);
  BOOST_CHECK(has_norm<B>::value);
  BOOST_CHECK(has_norm<C>::value);
  BOOST_CHECK(not has_norm<D>::value);
  BOOST_CHECK(has_norm<E>::value);
  BOOST_CHECK(not has_norm<int>::value);
  BOOST_CHECK(not has_norm<int*>::value);
  BOOST_CHECK(not has_norm<int()>::value);
  BOOST_CHECK(not has_norm<int(*)()>::value);
  BOOST_CHECK(not has_norm<int*(*)()>::value);
}

// Has value
BOOST_AUTO_TEST_CASE( test_has_value ) {
  BOOST_CHECK(not has_value<A>::value);
  BOOST_CHECK(has_value<B>::value);
  BOOST_CHECK(not has_value<C>::value);
  BOOST_CHECK(not has_value<D>::value);
  BOOST_CHECK(not has_value<E>::value);
  BOOST_CHECK(not has_value<int>::value);
  BOOST_CHECK(not has_value<int*>::value);
  BOOST_CHECK(not has_value<int()>::value);
  BOOST_CHECK(not has_value<int(*)()>::value);
  BOOST_CHECK(not has_value<int*(*)()>::value);
}

// Has norm type
BOOST_AUTO_TEST_CASE( test_as_norm_type ) {
  BOOST_CHECK(has_norm_type<A>::value);
  BOOST_CHECK(not has_norm_type<B>::value);
  BOOST_CHECK(not has_norm_type<C>::value);
  BOOST_CHECK(not has_norm_type<D>::value);
  BOOST_CHECK(not has_norm_type<E>::value);
  BOOST_CHECK(not has_norm_type<int>::value);
  BOOST_CHECK(not has_norm_type<int*>::value);
  BOOST_CHECK(not has_norm_type<int()>::value);
  BOOST_CHECK(not has_norm_type<int(*)()>::value);
  BOOST_CHECK(not has_norm_type<int*(*)()>::value);
}

// Has value type
BOOST_AUTO_TEST_CASE( test_has_value_type ) {
  BOOST_CHECK(has_value_type<A>::value);
  BOOST_CHECK(not has_value_type<B>::value);
  BOOST_CHECK(has_value_type<C>::value);
  BOOST_CHECK(not has_value_type<D>::value);
  BOOST_CHECK(not has_value_type<E>::value);
  BOOST_CHECK(not has_value_type<int>::value);
  BOOST_CHECK(not has_value_type<int*>::value);
  BOOST_CHECK(not has_value_type<int()>::value);
  BOOST_CHECK(not has_value_type<int(*)()>::value);
  BOOST_CHECK(not has_value_type<int*(*)()>::value);
}
